#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from random import randint, shuffle
import textwrap

from pyromaths import ex
from pyromaths.outils.Affichage import decimaux

precision = [u'au millième', u'au centième', u'au dixième', u'à l\'unité',
             u'à la dizaine', u'à la centaine', 'au millier',
             u'à la dizaine de milliers']

supinf = ['', u' par défaut', u' par excès']

class ArrondirNombreDecimal(ex.TexExercise):
    """Arrondir des nombres décimaux"""

    tags = ["Sixième"]

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        hasard = [valide_hasard() for dummy in range(4)]
        choix = [(i, j) for i in range(7)  for j in range(3)]
        shuffle(choix)
        self.choix_precision = [choix[i][0] for i in range(4)]
        self.choix_supinf = [choix[i][1] for i in range(4)]
    # FIXME
        # Arrondir n'est pas synonyme de valeur approchée
        # Valeur approchée par excès 
        # Valeur approchée par défaut 
        # Arrondi = la « meilleure » valeur approchée
        # et ne paraît employé ici correctement
        self.nombres = [hasard[0] / 10 ** (-self.choix_precision[0] + 4),
                        hasard[1] / 10 ** (-self.choix_precision[1] + 4),
                        hasard[2] / 10 ** (-self.choix_precision[2] + 4),
                        hasard[3] / 10 ** (-self.choix_precision[3] + 4)]

    def tex_statement(self):
        exo = ["\\exercice", '\\begin{enumerate}']
        for k in range(4):
            exo.append("\\item Arrondir " + decimaux(self.nombres[k]) + " " + 
                    precision[self.choix_precision[k]] + supinf[self.choix_supinf[k]] + 
                    '.')
        exo.append("\\end{enumerate}")
        return "\n".join(exo)

    def tex_answer(self):
        cor = ["\\exercice*", '\\begin{enumerate}']
        for k in range(4):
            arrondi = round(self.nombres[k], -self.choix_precision[k] + 3)
            if (arrondi > self.nombres[k]):
                defaut = arrondi - 10 ** (self.choix_precision[k] - 3)
                exc = arrondi
            if (arrondi <= self.nombres[k]):
                defaut = arrondi
                exc = arrondi + 10 ** (self.choix_precision[k] - 3)
            if (self.choix_supinf[k] == 0):
                solution = arrondi
            elif (self.choix_supinf[k] == 1):
                solution = defaut
            elif (self.choix_supinf[k] == 2):
                solution = exc
            cor.append('\\item L\'encadrement de ' + decimaux(self.nombres[k]) + ' ' + 
                    precision[self.choix_precision[k]] + ' est :\\par')
            cor.append(decimaux(defaut) + ' < ' + decimaux(self.nombres[k]) + ' < ' + 
                    decimaux(exc) + '\\par')
            cor.append(u'On en déduit que son arrondi ' + 
                    precision[self.choix_precision[k]] + ' ' + supinf[self.choix_supinf[k]] + 
                    ' est : ' + decimaux(solution) + '.')
        cor.append("\\end{enumerate}")
        return "\n".join(cor)

def valide_hasard():
    """renvoie un nombre float non multiple de 10000"""
    nbre, unite = randint(1000, 100000), randint(1, 9)
    return float(nbre) * 10 + unite



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

if __name__ == "__main__":
    import sys, os

    sys.path.append(os.path.join('..'))

# from .Racine import simplifie_racine, RacineDegre2, sqrt
from .Racine import RacineDegre2
from .Fractions import Fraction
from pyromaths.outils import Priorites3
import re
# from pyromaths.outils.Affichage import pTeX, TeX, radicalTeX, fTeX, Fractions, tTeX
from pyromaths.outils.Affichage import TeX, tTeX


class Polynome:
    '''Classe de polynôme pour le lycee'''

    def __init__(self, liste_coeff, var="x"):
        self.var = var  # Lettre pour la var
        liste_reduite = {}
        if isinstance(liste_coeff, str):
            liste_coeff = _str_Polynome(liste_coeff, var)
        elif isinstance(liste_coeff, list):
            liste_coeff = dict((i, liste_coeff[i]) for i in range(len(liste_coeff)))
        elif isinstance(liste_coeff, int) or isinstance(liste_coeff, Fraction) or isinstance(liste_coeff, float):
            liste_coeff = {0: liste_coeff}
        for i in list(liste_coeff.keys()):
            if liste_coeff[i] != 0:
                if isinstance(liste_coeff[i], (RacineDegre2, Fraction)):
                    # ===========================================================
                    #print(liste_coeff[i])
                    # ===========================================================
                    liste_reduite[i] = liste_coeff[i].simplifie()
                else:
                    liste_reduite[i] = Fraction(liste_coeff[i], 1)
        if liste_reduite == {} or liste_coeff == []:
            liste_reduite = {0: 0}
        self.dictio = liste_reduite
        self.puiss = list(liste_reduite.keys())
        self.puiss.sort(reverse=True)
        self.deg = self.degre()
        self.degre_max = max(0, self.deg)

    def __len__(self):
        return max(0, self.deg) + 1

    def degre(self):
        degre = float("-inf")
        for i in list(self.dictio.keys()):
            if i > degre and self.dictio[i] != 0:
                degre = i
        return degre

    def __getitem__(self, i):
        '''P[i] renvoie le coefficient de rang i'''
        return self.dictio.get(i, 0)

    def __str__(self):
        '''renvoie une str pour un affichage python'''
        return self.TeX(var=self.var)

    def __repr__(self):
        return '{classe}({coef}, var="{var}")'.format(
            classe=self.__class__.__name__,
            coef=repr(self.dictio),
            var=self.var,
        )

    def TeX(self, var='', display=True, parenthese=False):
        '''renvoie une chaine de caractere imprimant les fractions dans TeX'''
        if var == '':
            var = self.var
        exposants = [] + self.puiss
        premier = 1
        string = ''
        # =======================================================================
        # if display:
        #     fractex="\\dfrac"
        # else:
        #     fractex="\\frac"
        # =======================================================================
        for exposant in exposants:
            if premier:
                if self[exposant] == 1 and exposant != 0:
                    string = ''
                elif self[exposant] == -1 and exposant != 0:
                    string = '-'
                else:
                    string = TeX(self[exposant])
                premier = 0
            elif self[exposant] != 1:
                if self.dictio[exposant] == -1:
                    string += '-'
                else:
                    string += tTeX(self[exposant])
            else:
                string += "+"
            if exposant == 1:
                terme = var
            elif (exposant == 0) and ((self[exposant] == 1) or (self[exposant] == -1)):
                terme = '1'
            elif exposant == 0:
                terme = ''
            else:
                terme = var + u'^' + str(exposant)
            string += terme
        if parenthese and (len(self.puiss) > 1 or self[self.degre_max] < 0):
            return "\\left(" + string + "\\right)"
        else:
            return string

    def __add__(self, other):
        if isinstance(other, int) or isinstance(other, float) or isinstance(other, Fraction) or isinstance(other,
                                                                                                           RacineDegre2):
            return self + Polynome({0: other}, var=self.var)
        result = {0: 0}
        liste1 = self.puiss
        liste2 = other.puiss
        for i in liste1:
            s = self.dictio.get(i) + other.dictio.get(i, 0)
            if s != 0:
                result[i] = s
        for i in liste2:
            if i not in liste1:
                result[i] = other.dictio[i]
        return Polynome(result, var=self.var)

    def __mul__(self, other):
        if isinstance(other, Polynome):
            result = Polynome({0: 0}, var=self.var)
            for i in list(self.dictio.keys()):
                for j in list(other.dictio.keys()):
                    exposant = i + j
                    coefficient = self.dictio[i] * other.dictio[j]
                    result = result + Polynome({exposant: coefficient}, var=self.var)
            return result
        else:
            return self * Polynome(other)

    def __ne__(self, other):
        return not (self == other)

    def __eq__(self, other):
        if isinstance(other, int) or isinstance(other, Fraction) or isinstance(other, float):
            if self.degre_max == 0 and self[0] == other:
                return True
            else:
                return False
        elif self.var == other.var and self.dictio == other.dictio:
            return True
        else:
            return False

    def __pow__(self, other):
        if isinstance(other, int) and other >= 0:
            result = Polynome({0: 1}, self.var)
            for dummy in range(other):
                result *= self
            return result

    def __radd__(self, other):
        if isinstance(other, int) or isinstance(other, float) or isinstance(other, Fraction):
            return self + Polynome({0: other}, var=self.var)
        return self + other

    def __sub__(self, other):
        return self + (-other)

    def __rsub__(self, other):
        return -(self - other)

    def __neg__(self):
        return self * Polynome({0: -1}, var=self.var)

    def __rmul__(self, nombre):
        return self * Polynome({0: nombre})

    def __truediv__(self, other):
        if isinstance(other, int):
            return Fraction(1, other) * self
        elif isinstance(other, Fraction) or isinstance(other, float) or isinstance(other, RacineDegre2):
            return self // other
        else:
            quotient = Polynome({}, var=self.var)
            reste = self
            diviseur_degre = other.deg
            while diviseur_degre <= reste.deg:
                ajout_quotient_deg = reste.deg - diviseur_degre
                facteur = reste.dictio[reste.deg] / other.dictio[other.deg]
                if isinstance(facteur, float):
                    facteur = Fraction(reste.dictio[reste.deg], other.dictio[other.deg])
                ajout_quotient = Polynome({ajout_quotient_deg: facteur}, var=self.var)
                quotient = quotient + ajout_quotient
                soustrait_reste = ajout_quotient * other
                reste = reste - soustrait_reste
            return quotient, reste

    def simplifie(self):
        result = {}
        for i in self.puiss:
            if isinstance(self[i], (Fraction, RacineDegre2)):
                result[i] = self[i].simplifie()
            else:
                result[i] = self[i]
        return Polynome(result)

    def __call__(self, x, ):
        '''renvoie la Fraction ou l'int P(x)'''
        if x == float("inf") or x == float("-inf"):
            if self.degre_max == 0:
                return self[0]
            elif self.deg % 2 == 0:
                return self[self.deg] * float("inf")
            else:
                return self[self.deg] * x
        elif isinstance(x, str):
            return self.TeX(var=x)
        else:
            result = 0
            for i in list(self.dictio.keys()):
                result = result + self[i] * x ** i
            if isinstance(result, str):
                result = eval(Priorites3.priorites(result)[-1][0])
            if isinstance(result, (int, float)):
                return result
            elif isinstance(result, Fraction):
                result = result.simplifie()
                if isinstance(result, int):
                    return result
                if result.d == 1:
                    return result.n
                else:
                    return result
            elif result.denominateur == 1:
                return result.numerateur
            else:
                return result

    def racine_evidente(self, liste=[-2, -1, 0, 1, 2]):
        listerac = []
        for i in liste:
            if self(i) == 0:
                listerac = listerac + [i]
        return listerac

    def factorise(self, TeX=False, racines=[0, -1, 1, 2, -2]):
        facteurs = [Polynome({0: self.dictio[self.deg]}, var=self.var)]
        developpe, reste = self / facteurs[0]
        for r in racines:
            while developpe(r) == 0:
                rac = -r
                developpe, reste = developpe / Polynome({1: 1, 0: rac}, var=self.var)
                facteurs = facteurs + [Polynome({1: 1, 0: rac})]
        if TeX:
            stringTeX = ""
            if not (facteurs[0] == Polynome({0: 1}, var=self.var)):
                stringTeX += facteurs[0].TeX()
            for i in facteurs[1:]:
                if i[0] != 0:
                    stringTeX += "\\left(" + i.TeX() + "\\right) \\times "
                else:
                    stringTeX += i.TeX() + " \\times "
            if developpe == Polynome({0: 1}, var=self.var):
                return stringTeX[:-7]
            else:
                return stringTeX + "\\left(" + developpe.TeX() + "\\right)"
        else:
            return facteurs + [developpe]

    def derive(self):
        result = {}
        for i in list(self.dictio.keys()):
            if i == 0:
                result[0] = 0
            else:
                result[i - 1] = int(i) * self[i]
        return Polynome(result, self.var)

    def primitive(self):
        result = {}
        for i in list(self.dictio.keys()):
            result[i + 1] = Fraction(1, int(i + 1)) * self.dictio[i]
        return Polynome(result, self.var)


def _str_Polynome(string, var='x'):
    '''str -> dict'''
    # TODO reconnaitre les coefficients fractionnaires
    resultat = []
    termes = {}
    separeplus = (string).split("+")
    for element in separeplus:
        temp = []
        temp = element.split("-")
        for bis in temp:
            resultat.append(bis)
            if temp.index(bis) != len(temp) - 1:
                resultat.append(-1)
        if separeplus.index(element) != len(separeplus) - 1:
            resultat.append(1)
    if '' in resultat:
        resultat.remove('')
    for element in resultat:
        index = resultat.index(element)
        if index > 0:
            coeff = resultat[index - 1]
        else:
            coeff = 1
        if (element not in [-1, 1]):
            if element == var:
                termes[1] = (coeff)
            else:
                a = re.findall('\d+(?:\.\d*)?', element)
                if (len(a) == 1) and (not re.findall('\^', element)) and (re.findall(var, element)):
                    termes[1] = coeff * Fraction(eval(a[0]))
                elif (len(a) == 1) and (re.findall(var, element)):
                    termes[int(a[0])] = coeff * 1
                elif (len(a) == 1):
                    termes[0] = coeff * Fraction(eval(a[0]))
                else:
                    termes[int(a[1])] = coeff * Fraction(eval(a[0]))
    return termes



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import math
from random import randrange
from pyromaths.outils import Geometrie
from pyromaths import ex


def eq_droites(A, B):
    (xA, yA) = A
    (xB, yB) = B
    a = (yB - yA) / (xB - xA)
    b = (xB * yA - xA * yB) / (xB - xA)
    return (a, b)


def inter_droites(A, B, C, D):
    """
    Calcule les coordonnées du point d'intersection des droites (AB) et (CD)
    """

    (a1, b1) = eq_droites(A, B)
    (a2, b2) = eq_droites(C, D)
    if a1 == a2:  # droites parallèles
        xI = A[0]
        yI = A[1]
    else:
        xI = (b2 - b1) / (a1 - a2)
        yI = (a1 * b2 - a2 * b1) / (a1 - a2)
    return (xI, yI)


def dist_pt_droite(A, B, C):
    """
    calcule la distance du point C à la droite (AB)
    """

    (a, b) = eq_droites(A, B)
    (xC, yC) = C
    d = abs(a * xC - yC + b) / math.sqrt(a ** 2 + 1)
    return d


def dist_points(A, B):
    """ Calcul la distance entre deux points"""

    (xA, yA) = A
    (xB, yB) = B
    d = math.sqrt((xB - xA) ** 2 + (yB - yA) ** 2)
    return d


def coord_projete(A, B, C):
    """
    Calcule les coordonnées du projeté orthogonal de C sur la droite (AB)
    """

    (xA, yA) = A
    (xB, yB) = B
    (xC, yC) = C
    n = dist_points(A, B)
    p = (xB - xA) // n
    q = (yB - yA) // n
    s = p * (xC - xA) + q * (yC - yA)
    return (xA + s * p, yA + s * q)


def verifie_distance_mini(A, B, C, D):
    """
    Vérifie que la distance minimale entre [AB] et [AC] est supérieure à dmin
    """

    dmin = 1.2
    (xA, yA) = A
    (xB, yB) = B
    if xA > xB:
        (xA, yA, xB, yB) = (xB, yB, xA, yA)
    (xC, yC) = C
    (xD, yD) = D
    if xC > xD:
        (xC, yC, xD, yD) = (xD, yD, xC, yC)
    (xI, dummy) = inter_droites(A, B, C, D)
    if xA <= xI <= xB and xC <= xI <= xD or xA <= coord_projete(A, B, C)[0] <= \
        xB and dist_pt_droite(A, B, C) < dmin or xA <= coord_projete(A,
            B, D)[0] <= xB and dist_pt_droite(A, B, D) < dmin or xC <= \
        coord_projete(C, D, A)[0] <= xD and dist_pt_droite(C, D, A) < \
        dmin or xC <= coord_projete(C, D, B)[0] <= xD and dist_pt_droite(C,
            D, B) < dmin or dist_points(A, C) < dmin or dist_points(A, D) < \
        dmin or dist_points(B, C) < dmin or dist_points(B, D) < dmin:
        isValid = False
    else:
        isValid = True
    return isValid


def verifie_angle(lpoints, A, B, C):
    """
    Vérifie que l'angle BAC ne coupe pas les autres angles déjà tracés
    """

    if len(lpoints) == 0:  # Premier angle créé
        isValid = True
    else:
        for i in range(len(lpoints)):
            (A1, B1, C1) = (lpoints[i])[:3]
            isValid = verifie_distance_mini(A, B, A1, B1) and \
                verifie_distance_mini(A, B, A1, C1) and \
                verifie_distance_mini(A, C, A1, B1) and \
                verifie_distance_mini(A, C, A1, C1)
            if not isValid:
                break
    return isValid


def cree_angles(nb_angles, xmax, ymax):
    '''
    crée une série d\'angles "non séquents"
    '''

    (xmax, ymax) = (xmax - .5, ymax - .5)  # taille de l'image en cm
    lg_seg = 6  # longueur des côtés des angles
    lpoints = []
    cpt = 0  # evite une boucle infinie
    while len(lpoints) < nb_angles and cpt < 1000:
        (xA, yA) = (randrange(5, xmax * 10) / 10, randrange(5, ymax * 10) / 10)
        alpha = randrange(360)  # angle entre un côté et l'horizontal
        if len(lpoints) < nb_angles // 2:
            beta = randrange(90, 180)  # crée un angle droit ou obtus
        else:
            beta = randrange(0, 75) + 15  # crée un angle aigu (entre 15° et 89°)
        xB = xA + lg_seg * math.cos((alpha * math.pi) / 180)
        yB = yA + lg_seg * math.sin((alpha * math.pi) / 180)
        xC = xA + lg_seg * math.cos(((alpha + beta) * math.pi) / 180)
        yC = yA + lg_seg * math.sin(((alpha + beta) * math.pi) / 180)
        (A, B, C) = ((xA, yA), (xB, yB), (xC, yC))
        if xA != xB and xA != xC and .5 < xB < xmax and .5 < yB < ymax and \
            .5 < xC < xmax and .5 < yC < ymax and verifie_angle(lpoints,
                A, B, C):
            lpoints.append((A, B, C, alpha, beta))
        else:
            cpt = cpt + 1

    # print len(lpoints)

    return lpoints


def PosAngle(alpha, beta):
    """retourne les angles pour placer les points sur la figure"""

    A = (alpha + beta / 2 + 180) % 360
    B = (alpha - 90) % 360
    C = (alpha + beta + 90) % 360
    return (A, B, C)


def PointName(l3noms, indice):
    liste = []
    for i in range(3):
        liste.append(l3noms[i])
    return tuple(liste)


def figure(exo, cor, lpoints, lnoms, xmax, ymax):
    exo.append("\\begin{pspicture}(%s,%s)" % (xmax, ymax))
    exo.append("\\psframe(0,0)(%s,%s)" % (xmax, ymax))
    exo.append("\\psset{PointSymbol=none,MarkAngleRadius=0.6}")
    cor.append("\\begin{pspicture}(%s,%s)" % (xmax, ymax))
    cor.append("\\psset{PointSymbol=none,MarkAngleRadius=0.6}")
    cor.append("\\psframe(0,0)(%s,%s)" % (xmax, ymax))
    for i in range(len(lnoms)):
        points_exo = ''
        points_cor = ''
        points_exo += "\\pstGeonode[PointName={%s,%s,%s}," % lnoms[i]
        points_cor += "\\pstGeonode[PointName={%s,%s,%s}," % lnoms[i]
        points_exo += "PosAngle={%s,%s,%s}]" % PosAngle(lpoints[i][3], lpoints[i][4])
        points_cor += "PosAngle={%s,%s,%s}]" % PosAngle(lpoints[i][3], lpoints[i][4])
        for j in range(3):
            points_exo += "(%.2f,%.2f)" % lpoints[i][j]
            points_exo += "{a%s%s}" % (j, i)
            points_cor += "(%.2f,%.2f)" % lpoints[i][j]
            points_cor += "{a%s%s}" % (j, i)
        exo.append(points_exo)
        cor.append(points_cor)
        exo.append("\\pstMarkAngle{a%s%s}{a%s%s}{a%s%s}{}" % (1, i, 0, i, 2, i))
        cor.append("\\pstMarkAngle{a%s%s}{a%s%s}{a%s%s}{}" % (1, i, 0, i, 2, i))
        exo.append("\\pstLineAB[nodesepB=-.5]{a0%s}{a1%s}\\pstLineAB[arrows=-|,linestyle=none]{a0%s}{a1%s}" % (i, i, i, i))
        cor.append("\\pstLineAB[nodesepB=-.5]{a0%s}{a1%s}\\pstLineAB[arrows=-|,linestyle=none]{a0%s}{a1%s}" % (i, i, i, i))
        exo.append("\\pstLineAB[nodesepB=-.5]{a0%s}{a2%s}\\pstLineAB[arrows=-|,linestyle=none]{a0%s}{a2%s}" % (i, i, i, i))
        cor.append("\\pstLineAB[nodesepB=-.5]{a0%s}{a2%s}\\pstLineAB[arrows=-|,linestyle=none]{a0%s}{a2%s}" % (i, i, i, i))
    exo.append("\\end{pspicture}\\par")
    cor.append("\\end{pspicture}\\par")
    return (exo, cor)


def reponses(exo, cor, lpoints, lnoms):
    cor.append("\\begin{multicols}{4}")
    for i in range(len(lnoms)):
        cor.append("$\\widehat{%s%s%s}=%s\degres$\\par" % (lnoms[i][1],
                 lnoms[i][0], lnoms[i][2], lpoints[i][4]))
        if lpoints[i][4] < 90:
            cor.append("angle aigu\\par")
        elif lpoints[i][4] > 90:
            cor.append("angle obtus\\par")
        else:
            cor.append("angle droit\\par")
    cor.append("\\end{multicols}")
    exo.append("\\begin{tabularx}{\\textwidth}{|*{4}{X|}}")
    exo.append("\\hline angle 1 : & angle 2 : & angle 3 : & angle 4 : \\\\")
    exo.append("\\hline &&& \\\\ &&& \\\\ &&& \\\\ \\hline")
    exo.append("\\end{tabularx}")


def _MesureAngles():
    nb_angles = 4
    (xmax, ymax) = (18, 8)  # taille de l'image en cm
    lnoms = []
    lpoints = []
    cpt = 0
    while len(lpoints) < nb_angles:
        if cpt > 1000:
            lpoints = []
            cpt = 0
        lpoints = cree_angles(nb_angles, xmax, ymax)
        cpt = cpt + 1
    tmpl = Geometrie.choix_points(3 * nb_angles)
    for i in range(nb_angles):
        lnoms.append(tuple(tmpl[3 * i:3 * i + 3]))
    exo = ["\\exercice", "Nommer, mesurer et donner la nature de chacun des angles suivants :\\par "]
    cor = ["\\exercice*", "Nommer, mesurer et donner la nature de chacun des angles suivants :\\par "]
    figure(exo, cor, lpoints, lnoms, xmax, ymax)
    reponses(exo, cor, lpoints, lnoms)
    return (exo, cor)

class MesureAngles(ex.LegacyExercise):
    """Mesurer des angles"""

    tags = ["Sixième"]
    function = _MesureAngles

class ConstruireZigZag(ex.TexExercise):
    """Construire des angles"""

    tags = ["Sixième"]

    def __init__(self, *args, **kwargs):
        """ Crée une liste de nbp points situés à la distance lg les uns des
        autres"""
        super().__init__(*args, **kwargs)

        from pyromaths.outils.Conversions import radians
        from math import sin, cos
        self.lg, nbp = 4, 6
        fini = False
        while not fini:
            ar = randrange(80, 91)
            angles_relatifs = [ar]
            angles_absolus = [ar]
            ar = radians(ar)
            points = [(.2, .2), (.2 + self.lg * cos(ar), .2 + self.lg * sin(ar))]
            for i in range(nbp - 1):
                point = (-1, -1)
                cpt = 0  # évite les boucles infinies
                while (not point[0] < 16.2 or not 0.2 < point[1] < self.lg + 1) and cpt < 100:
                    if i % 2:
                        aa = randrange(angles_absolus[-1], 80)
                    else:
                        aa = randrange(-80, angles_absolus[-1])
                    ar = 180 - abs(aa - angles_absolus[-1])
                    aar = radians(aa)
                    point = (points[-1][0] + self.lg * cos(aar), points[-1][1] + self.lg * sin(aar))
                    cpt += 1
                if cpt == 100:
                    break
                else:
                    points.append(point)
                    angles_absolus.append(aa)
                    angles_relatifs.append(ar)
            if cpt < 100: fini = True
        self.points, self.angles_relatifs, self.angles_absolus = points, angles_relatifs, angles_absolus

    def tex_place_les_points_zigzag(self, corrige=False):
        exo = "\\pstGeonode[PosAngle=%.2f, PointSymbol=x](%.2f, %.2f){%s} " % \
                (self.angles_absolus[0] - 180, self.points[0][0], self.points[0][1], chr(65))
        if not corrige:
            exo += "\\pstGeonode[PosAngle=%.2f, PointSymbol=x](%.2f, %.2f){B} " % \
                (self.angles_absolus[1] + self.angles_relatifs[1] / 2 - 180, self.points[1][0], self.points[1][1])
            exo += "\pstSegmentMark{A}{B}\n"
        x1, y1 = inter_droites(self.points[0], self.points[-1], self.points[1], self.points[-2])
        if 0 < x1 < 18 and 0 < y1 < self.lg + 2:
            cas = 0
        else:
            x1, y1 = inter_droites(self.points[0], self.points[-2], self.points[1], self.points[-1])
            cas = 1
        for i in range(1, 5):
            exo += "\pscircle[linecolor=Gray](%.2f, %.2f){%.1f}\n" % (x1, y1, i / 10)
        if corrige:
            for i in range(1, len(self.angles_relatifs)):
                if self.angles_absolus[i - 1] > self.angles_absolus[i]:
                    exo += "\\pstGeonode[PosAngle=%.2f](%.2f, %.2f){%s} " % \
                            (self.angles_absolus[i] - self.angles_relatifs[i] / 2 - 180, self.points[i][0], self.points[i][1], chr(i + 65))
                else:
                    exo += "\\pstGeonode[PosAngle=%.2f](%.2f, %.2f){%s} " % \
                            (self.angles_absolus[i] + self.angles_relatifs[i] / 2 - 180, self.points[i][0], self.points[i][1], chr(i + 65))

                exo += "\pstSegmentMark{%s}{%s}\n" % (chr(i + 64), chr(i + 65))
            exo += "\\pstGeonode[PosAngle=%.2f, PointSymbol=x](%.2f, %.2f){%s} " % \
                    (self.angles_absolus[-1], self.points[-1][0], self.points[-1][1], chr(len(self.points) + 64))
            exo += "\pstSegmentMark{%s}{%s}\n" % (chr(len(self.points) + 63), chr(len(self.points) + 64))
            for i in range(len(self.angles_relatifs) - 1):
                if self.angles_absolus[i] > self.angles_absolus[i + 1]:
                    if self.angles_relatifs[i + 1] == 90:
                        exo += "\\pstRightAngle{%s}{%s}{%s}\n" % \
                                (chr(i + 65), chr(i + 66), chr(i + 67))
                    else:
                        exo += "\\pstMarkAngle{%s}{%s}{%s}{%s\\degres}\n" % \
                                (chr(i + 65), chr(i + 66), chr(i + 67), self.angles_relatifs[i + 1])
                else:
                    if self.angles_relatifs[i + 1] == 90:
                        exo += "\\pstRightAngle{%s}{%s}{%s}\n" % \
                                (chr(i + 67), chr(i + 66), chr(i + 65))
                    else:
                        exo += "\\pstMarkAngle{%s}{%s}{%s}{%s\\degres}\n" % \
                                (chr(i + 67), chr(i + 66), chr(i + 65), self.angles_relatifs[i + 1])
            if cas:
                exo += "\psline[linestyle=dotted](B)(G) "
                exo += "\psline[linestyle=dotted](A)(F)"
            else:
                exo += "\psline[linestyle=dotted](B)(F) "
                exo += "\psline[linestyle=dotted](A)(G)"
        return exo

    def tex_commun(self):
        exo = [u'Construire sur la figure ci-dessous les points $C$, $D$, $E$, $F$ et $G$ pour obtenir un zigzag tel que :\\par']
        exo_t = '$'
        for i in range(len(self.angles_relatifs) - 1):
            exo_t += r"\widehat{%s%s%s}=%s\degres \qquad " % (chr(i + 65), chr(i + 66), chr(i + 67), self.angles_relatifs[i + 1])
        exo_t += r'$\par'
        exo.append(exo_t)
        exo.append(u"Quand le travail est fait avec une bonne précision, les ")
        x1, y1 = inter_droites(self.points[0], self.points[-1], self.points[1], self.points[-2])
        if 0 < x1 < 18 and 0 < y1 < self.lg + 2:
            exo.append(u"droites $(AG)$ et $(BF)$ se coupent au c\\oe ur de la cible.\\par")
        else:
            exo.append(u"droites $(AF)$ et $(BG)$ se coupent au c\\oe ur de la cible.")
        exo.append(r'\begin{center}')
        exo.append("\\fbox{\n\\begin{pspicture}(-.4,-.4)(16.4, %s)\n" % (self.lg + 1.5))
        return exo


    def tex_statement(self):
        exo = [r'\exercice']
        exo.append(u'Voici deux exemples de zigzags :\par')
        exo.append(r'\psset{unit=3.5mm,PointSymbol=none}')
        exo.append(r'\begin{pspicture}(-.4,-1)(16.4, 7.5)')
        exo.append(r'%\psgrid')
        exo.append(r'\pstGeonode[PosAngle=-96.00,PointSymbol=x](0.20, 0.20){A}')
        exo.append(r'\pstGeonode[PosAngle=-265.00](0.83, 6.17){B} \pstSegmentMark{A}{B}')
        exo.append(r'\pstGeonode[PosAngle=-87.50](2.48, 0.40){C} \pstSegmentMark{B}{C}')
        exo.append(r'\pstGeonode[PosAngle=-241.00](3.63, 6.29){D} \pstSegmentMark{C}{D}')
        exo.append(r'\pstGeonode[PosAngle=-90.00](9.23, 4.14){E} \pstSegmentMark{D}{E}')
        exo.append(r'\pstGeonode[PosAngle=-299.50](14.83, 6.29){F} \pstSegmentMark{E}{F}')
        exo.append(r'\pstGeonode[PosAngle=-80.00,PointSymbol=x](15.87, 0.38){G}')
        exo.append(r'\pstSegmentMark{F}{G}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{A}{B}{C}{}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{D}{C}{B}{}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{C}{D}{E}{}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{F}{E}{D}{}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{E}{F}{G}{}')
        exo.append(r'\end{pspicture}')
        exo.append(r'\hspace{2cm}')
        exo.append(r'\begin{pspicture}(-.4,-1)(16.4, 7.5)')
        exo.append(r'\pstGeonode[PosAngle=-91.00,PointSymbol=x](0.20, 0.20){A}')
        exo.append(r'\pstGeonode[PosAngle=-263.50](0.30, 6.20){B} \pstSegmentMark{A}{B}')
        exo.append(r'\pstGeonode[PosAngle=-101.00](1.76, 0.38){C} \pstSegmentMark{B}{C}')
        exo.append(r'\pstGeonode[PosAngle=-259.00](5.28, 5.23){D} \pstSegmentMark{C}{D}')
        exo.append(r'\pstGeonode[PosAngle=-82.50](10.37, 2.05){E} \pstSegmentMark{D}{E}')
        exo.append(r'\pstGeonode[PosAngle=-286.00](14.46, 6.44){F} \pstSegmentMark{E}{F}')
        exo.append(r'\pstGeonode[PosAngle=-79.00,PointSymbol=x](15.61, 0.55){G}')
        exo.append(r'\pstSegmentMark{F}{G}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{A}{B}{C}{}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{D}{C}{B}{}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{C}{D}{E}{}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{F}{E}{D}{}')
        exo.append(r'\pstMarkAngle[MarkAngleRadius=1.5]{E}{F}{G}{}')
        exo.append(r'\end{pspicture}\par')
        exo.append(r'\psset{unit=1cm}')
        exo.extend(self.tex_commun())
        exo.append(self.tex_place_les_points_zigzag(corrige=False))
        exo.append(r'\end{pspicture}')
        exo.append('}\n\\end{center}')
        return "\n".join(exo)

    def tex_answer(self):
        exo = [r'\exercice*']
        exo.extend(self.tex_commun())
        exo.append(self.tex_place_les_points_zigzag(corrige=True))
        exo.append(r'\end{pspicture}')
        exo.append('}\n\\end{center}')
        return "\n".join(exo)




#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

from decimal import Decimal
import random
from pyromaths.outils import Affichage, Arithmetique
from pyromaths.ex import LegacyExercise
#===============================================================================
# Écrire un nombre en lettres ou en chiffres
#===============================================================================


def NombreEnLettres(n, France=True):
    unite = {
        1: "un",
        2: 'deux',
        3: 'trois',
        4: 'quatre',
        5: 'cinq',
        6: 'six',
        7: 'sept',
        8: 'huit',
        9: 'neuf',
        10: 'dix',
        11: 'onze',
        12: 'douze',
        13: 'treize',
        14: 'quatorze',
        15: 'quinze',
        16: 'seize',
        17: 'dix-sept',
        18: 'dix-huit',
        19: 'dix-neuf',
        }
    dizaineF = {2: 'vingt', 3: 'trente', 4: 'quarante', 5: 'cinquante',
                6: 'soixante', 7: "", 8: 'quatre-vingt', 9: ""}
    dizaineB = {2: 'vingt', 3: 'trente', 4: 'quarante', 5: 'cinquante',
                6: 'soixante', 7: 'septante', 8: 'octante', 9: 'nonante'}
    coefs = {0: 'cent', 1: 'mille', 2: 'million', 3: 'milliard'}
    result = ""

    # Cas particulier de zéro
    if n == 0:
        result = u'zéro'
    else:
        coef = 0

        while n > 0:

        # Récupération de l'unité du bloc de trois chiffres en cours

            u = n % 10
            n = n // 10

        # Récupération de la dizaine du bloc de trois chiffres en cours

            d = n % 10
            n = n // 10
        # Traitement des dizaines

            temp = ""

        # Passage sur la dizaine inférieure pour 10 à 19
        # et pour 70-79 90-99 dans le cas de la France

            if d == 1 or (d == 7 or d == 9) and France:
                d = d - 1
                u = u + 10
            if d > 1:
                if France:
                    if n:
                        temp = '-' + dizaineF[d]
                    else:
                        temp = dizaineF[d]

                    # Ajout du cas particulier de 'et' entre la dizaine et 1

                    if d < 8 and (u == 1 or u == 11):
                        temp = temp + '-et'
                else:
                    if n:
                        temp = '-' + dizaineB[d]
                    else:
                        temp = dizaineB[d]

                    # Ajout du cas particulier de 'et' entre la dizaine et 1

                    if u == 1:
                        temp = temp + '-et'

        # ajout du texte de l'unité

            if u > 0 and (d or n):
                temp = temp + '-' + unite[u]
            elif u > 0:
                temp = unite[u]

        # ajout du 's' à Quatre-vingt si rien ne suit
        # if (result == '') and (d == 8) and (u == 0) and France : result = 's'

            if d == 8 and u == 0 and France:
                temp = temp + 's'
            result = temp + result


            # Récupération de la centaine du bloc de trois chiffres en cours

            c = n % 10
            n = n // 10
            if c > 0:
                temp = ""
                if c > 1 and n:
                    temp = '-' + unite[c]
                elif c > 1:
                    temp = unite[c]
                if c == 1 and not n:
                    temp = coefs[0]
                else:
                    temp = temp + '-' + coefs[0]

                # Traitement du cas particulier du 's' à cent si rien ne suit

                if result == "" and c > 1:
                    result = 's'
                result = temp + result

            # Traitement du prochain groupe de 3 chiffres

            if n > 0:
                coef = coef + 1
                i = n % 1000
                if i > 1 and coef > 1:
                    result = 's' + result

                # Traitement du cas particulier 'mille' ( non pas 'un mille' )

                if i == 1 and coef == 1:
                    n = n - 1
                    result = coefs[coef] + result
                elif i > 0:

                    result = '-' + coefs[coef] + result
    return result

def nombreATrouver():
    """a contient la liste des nombres à créer où il peut ne pas y avoir de
    centaines, de centaines de milliers, d'unités ou de milliers"""
    from decimal import Decimal
    a = [random.randrange(100) + random.randrange(1000) * 10 ** 3,
         random.randrange(1000) + random.randrange(100) * 10 ** 3,
         random.randrange(1000) * 10 ** 3, random.randrange(1000)]
    (lnb, liste) = ([], [])
    for i in range(4):
        lnb.append(random.randrange(1, 1000) * 10 ** 6 + a[i])
    for i in range(4):
        n = a[i]
        if n % 1000:  # il y a des unités dans le nombre
            e = random.randrange(1, 4)
            lnb.append(n / Decimal(10 ** (e)))
        else:
            e = random.randrange(4, 7)
            lnb.append(n / Decimal(10 ** (e)))
    for i in range(8):
        liste.append(lnb.pop(random.randrange(len(lnb))))
    return liste

def EcritNombreDecimal(n):
    txt = ""
    if n != int(n):
        # n n'est pas un nombre entier
        (e, d) = str(n).split('.')
        (e, d) = (int(e), int(d))
    else:
        (e, d) = (int(n), 0)
    if not d:
        txt = NombreEnLettres(e)
    elif e:
        txt = NombreEnLettres(e)
    if d:
        partieDec = [u" dixième", u" centième", u" millième"]
        if txt.rfind("un") == len(txt) - 2:

        # le texte se finit par un. On l'accorde en genre avec unité

            txt = txt + "e"
        if e == 1:
            txt = txt + u' unité et '
        if e > 1:
            txt = txt + u' unités et '
        txt = txt + NombreEnLettres(d) + partieDec[len(str(n).split('.')[1]) - 1]
        if d > 1:
            txt = txt + 's'
    return txt

def EcritEnChiffre(exo, cor):
    lnb = nombreATrouver()
    for i in range(len(lnb)):
        exo.append("\\item " + EcritNombreDecimal(lnb[i]) + 
                 " : \\dotfill")
        cor.append("\\item " + EcritNombreDecimal(lnb[i]) + " : ")
        cor.append(Affichage.decimaux(lnb[i], 0) + '')


def EcritEnLettre(exo, cor):
    lnb = nombreATrouver()
    for i in range(8):
        exo.append("\\item " + Affichage.decimaux(lnb[i], 0) + 
                 " : \\dotfill")
        cor.append("\\item " + Affichage.decimaux(lnb[i], 0) + 
                " : ")
        cor.append(EcritNombreDecimal(lnb[i]) + '')


def _EcrireNombreLettre():
    exo = ["\\exercice", "\\begin{enumerate}",
            u'\\item Écrire en chiffres les nombres suivants.',
            '\\begin{enumerate}']
    cor = ["\\exercice*", "\\begin{enumerate}",
            u'\\item Écrire en chiffres les nombres suivants.',
            '\\begin{enumerate}']

    EcritEnChiffre(exo, cor)

    exo.append('\\end{enumerate}')
    exo.append(u'\\item Écrire en lettres les nombres suivants (sans utiliser le mot ``virgule").')
    exo.append('\\begin{enumerate}')
    cor.append('\\end{enumerate}')
    cor.append(u'\\item Écrire en lettres les nombres suivants (sans utiliser le mot ``virgule").')
    cor.append('\\begin{enumerate}')

    EcritEnLettre(exo, cor)

    exo.append('\\end{enumerate}')
    exo.append('\\end{enumerate}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{enumerate}')
    return (exo, cor)

class EcrireNombreLettre(LegacyExercise):
    """Écrire un nombre décimal"""

    tags = ["Sixième"]
    function = _EcrireNombreLettre


#===============================================================================
# Conversions
#===============================================================================

units = ["L", "m", "g"]
division = ["k", "h", "da", "", "d", "c", "m"]

# paramétrage des flèches : mofifie le paramétrage par défaut de PSTricks  s'il n'est pas limité par un environnement ou {groupe}
# # nodesepA = -1.5mm  : décale le départ de la flèche
# # linewidth = 0.6pt  : épaisseur de la flèches
# # linestyle = dotted : style pointillé
# # vref = -0.8mm      : décale la flèche vers le bas, sous les chiffres
PSSET_FLECHE = '\\psset{nodesepA = -1.5mm, linewidth = 0.6pt, linestyle = dotted, vref = -0.8mm}'


def valeurs_units():
    """
    renvoie les valeurs pour les conversions d'unités
    """

    a = Arithmetique.valeur_alea(101, 999)
    p = random.randrange(-2, 0)
    unit = random.randrange(3)
    if unit:
        # mètres ou grammes, on peut utiliser les k
        imax = 7
    else:
        # Litres, donc pas de kL
        imax = 6

    div0 = random.randrange(imax + p)

    while 1:
        div1 = random.randrange(imax)
        if div0 != div1:
            break

    if not unit:  # Litres, donc pas de kL donc on décale d'un rang
        div0, div1 = div0 + 1, div1 + 1

    return (a, p, unit, div0, div1)
        # 101< a <999 ex a = 245
        # p = {-2,-1} donne 2,45 ou 24,5
        # unit = {0, 1, 2} => {L, m, g}
        # div0 unité 0
        # div1 unité converti

def _tex_units():
    """
    Écrit l'exercice sur les conversions d'unités et le corrigé au format
    LaTeX
    @param exo: fichier exercices
    @param cor: fichier corrige
    """

    exo = ["\\exercice", 'Effectuer les conversions suivantes :',
            '\\begin{multicols}{3}\\noindent', '\\begin{enumerate}']
    cor = ["\\exercice*",
            # paramétrage des flèches, ce paramétrage est limité à l'exercice
            # et ne modifie pas le paramétrage PSTricks du document car sa portée est limité par le groupe ouvert par "{"
           "{",
            PSSET_FLECHE,
           'Effectuer les conversions suivantes :',
            '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']

    # Construit les 6 questions de l'exercice
    for i in range(6):
        (a, p, unit, div0, div1) = valeurs_units()
        if unit:
            u = tuple([units[unit] for i in range(7)])
        else:
            u = tuple([units[unit] for i in range(6)])
        nb0 = Affichage.decimaux(a * Decimal(10) ** p, 0)
        nb1 = Affichage.decimaux(a * Decimal(10) ** ((p + div1) - div0),
                0)
        exo.append("\\item %s~%s%s=\dotfill~%s%s" % (nb0, division[div0],
                 units[unit], division[div1], units[unit]))
        cor.append("\\item %s~%s%s=%s~%s%s\\par" % (nb0, division[div0],
                 units[unit], nb1, division[div1], units[unit]))
        nblist = [nb0[i] for i in range(len(nb0))]
        if nblist.count(','):
            chf_unite = nblist.index(',') - 1
            nblist.pop(chf_unite + 1)
        else:
            chf_unite = len(nblist) - 1

        tex_tableau(cor, div0, div1, u, nblist, chf_unite)

        cor.append("\\end{tabular}")
        cor.append("\\ncline{->}{virg0}{virg1}")

    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    # ferme le groupe limitant la portée de PSSET_FLECHE
    cor.append('{')
    return (exo, cor)

class tex_units(LegacyExercise):
    """Conversions unités"""

    tags = ["Sixième"]
    function = _tex_units


def tex_tableau(cor, div0, div1, u, nblist, chf_unite):
    """tableau de conversion pour les unités simples : L, g ou m"""

    # Si len(u) == 6, on a des Litres, on ne doit pas avoir la colonne kL
    if len(u) == 6:
        cor.append("\\begin{tabular}{c|c|c|c|c|c}")
        cor.append("h%s & da%s & %s & d%s & c%s & m%s \\\\ \\hline" % u)
        # décale d'une colonne pour supprimer kL
        delta = 1
        div0 = div0 - 1
        div1 = div1 - 1

    else:
        cor.append("\\begin{tabular}{c|c|c|c|c|c|c}")
        cor.append("k%s & h%s & da%s & %s & d%s & c%s & m%s \\\\ \\hline" % u)
        # ne supprime pas le kg, km
        delta = 0
    for dummy in range(-div0 + chf_unite):
        tmp = nblist.pop(0)
        nblist[0] = tmp + nblist[0]

    for dummy in range(div0 - chf_unite):
        nblist.insert(0, '0')


    for dummy in range(-len(u) + len(nblist)):
        tmp = nblist.pop(7)
        nblist[6] = nblist[6] + tmp

    # les zéros à droites des chiffres significatifs
    for dummy in range(len(u) - len(nblist)):
        nblist.append('0')

    # place les \nodes et la virgule dans le tableau
    nblist[div0] = "%s\\Rnode{virg0}{\\ }" % (nblist[div0])
    nblist[div1] = "{%s\\Rnode{virg1}{\\textcolor{red}{ \\LARGE ,}}}" % (nblist[div1])

    # ajoute au tabular la ligne avec 6 ou 7 colonnes
    cor.append(("%s " + ("& %s"*(6 - delta))) % tuple(nblist))



def exo_conversion(exposant):
    """construit l'exercice de conversion d'unité d'aire ou de volume
    exposant 2 pour m²
    exposant 3 pour m³"""

    exo = ["\\exercice", 'Effectuer les conversions suivantes :',
            '\\begin{multicols}{3}\\noindent', '\\begin{enumerate}']
    cor = ["\\exercice*",
            # la portée de \psset est par le group ouvert par "{"
            "{",
            PSSET_FLECHE,
            '\\def\\virgule{\\textcolor{red}{ \\LARGE ,}}',
            'Effectuer les conversions suivantes :',
            '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']

    # ajoute le ² ou ³ si nécessaire
    str_exposant = (u"^%s" % (exposant)) * (exposant > 1)

    u = tuple([division[i] + "m%s" % str_exposant for i in range(7)])
    entete_tableau = ((" \\multicolumn{%s}{c|}" % exposant + "{$\\rm %s$} &") * 6 + "\\multicolumn{%s}{c}" % exposant + "{$\\rm %s$}") % u
    ligne_tab = []

    for i in range(6):
        # imprime la correction et sauvegarde la ligne et la flèche pour le tableau imprimé ensuite
        ligne_tab += tex_conversion(exo, cor, exposant, u) + ["\\ncline{->}{virg0}{virg1} \\\\"]

    # ferme la correction et l'énoncé
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')

    # impression du tableau et des lignes sauvegardées précédemment
    cor.append("\\begin{tabular}{*{%s}{p{3.5mm}|}p{3.5mm}}" % (exposant * 7 - 1))
    cor.append(entete_tableau + "\\\\ \\hline")
    # ajoute les lignes affichant les conversions
    cor += ligne_tab
    cor.append("\\end{tabular}")
    # ferme le groupe dans lequel PSSET_FLECHE portait
    cor.append("}")
    # C'est fini
    return (exo, cor)

def _exo_conversion_2d():
    return exo_conversion(2)

class exo_conversion_2d(LegacyExercise):
    """Conversions unités d'aires"""

    tags = ["Sixième"]
    function = _exo_conversion_2d

def _exo_conversion_3d():
    return exo_conversion(3)

class exo_conversion_3d(LegacyExercise):
    """Conversions unités de volumes"""

    tags = ["Sixième"]
    function = _exo_conversion_3d


def tex_conversion(exo, cor, exposant, u):
    """Écrit une question sur les conversions d'unités d'aires ou de volume
    et le corrigé au format LaTeX
    @param exo: fichier exercices
    @param cor: fichier corrige
    exposant = 2 ou 3 pour les aires ou les volumes
    """

    a = random.randint(101, 999)
    p = random.randint(-2, -1)
    while True:
        (div0, div1) = (random.randrange(6), random.randrange(7),)
        # Pas de mm³ par ce que ça sort du tableau
        if (div0 - div1) in [-2, -1, 1, 2]:
            # pas trop loin car ça fait de très longs nombres
            break
    nb0 = a * Decimal(10) ** p
    nb1 = nb0 * Decimal(10) ** (exposant * (div1 - div0))

    exo.append("\\item $\\unit[%s]{%s}=\\unit[\\dotfill]{%s}$" % 
            (Affichage.decimaux(nb0), u[div0], u[div1]))
    cor.append("\\item $\\unit[%s]{%s}=\\unit[%s]{%s}$\\vspace{1ex}\\par" % 
            (Affichage.decimaux(nb0), u[div0],
                Affichage.decimaux(nb1), u[div1]))

    return tex_tableau_conversion(div0, div1, nb0, u, exposant)


def tex_tableau_conversion(div0, div1, nb0, u, exposant):
    nb_dict = nbre_to_dict(nb0, div0, div1, exposant)
    nblist = [str(nb_dict.get(i, "")) for i in range(7 * exposant)]
    nblist[exposant * (div0 + 1) - 1] = "%s\\Rnode{virg0}{\\ }" % nb_dict.get(exposant * (div0 + 1) - 1, "0")
    nblist[exposant * (div1 + 1) - 1] = "{%s\\Rnode{virg1}{\\virgule}}" % nb_dict.get(exposant * (div1 + 1) - 1, "0")
    return [("%s " + "& %s"*(7 * exposant - 1)) % tuple(nblist)]


def nbre_to_dict(nbre , div0, div1, exposant):
    # exposant peut être 2 ou 3 pour les m² ou les m³
    nbre = int(round(nbre * 100))
    nb_dict = {}
    for i in range(min(exposant * (div0 + 1), exposant * (div1 + 1)) - 1, max(exposant * (div0 + 1), exposant * (div1 + 1))):
            nb_dict[i] = "\\textcolor{red}{0}"
    curseur = 1 + exposant * (div0 + 1)
    while nbre % 10 == 0:
        nbre = nbre // 10
        curseur -= 1
    while nbre > 0:
        chiffre = nbre % 10
        nbre = (nbre - chiffre) // 10
        nb_dict[curseur] = "\\textcolor{blue}{%s}" % chiffre
        curseur -= 1
    return nb_dict


#===============================================================================
# Placer une virgule
#===============================================================================

valeurs = ["milliers", "centaines", "dizaines", u"unités",
           u"dixièmes", u"centièmes", u"millièmes"]


def valeurs_decimaux():
    """
    Choisit les valeurs
    """

    nb = 0
    chiffres = [1, 2, 3, 4, 5, 6, 7, 8, 9]
    for i in range(6):
        nb = nb + chiffres.pop(random.randrange(len(chiffres))) * 10 ** \
            i
    return nb


def tex_place_virgule(exo, cor):
    """
    Écrit un exercices demandant de placer une virgule dans un nombre.
    @param exo: fichier exerices
    @param cor:fichier corrigé
    """

    valeurs_index = [0, 1, 2, 3, 4, 5, 6]
    nb = valeurs_decimaux()
    exo.append(u"Placer une virgule (en ajoutant éventuellement des zéros) dans\
            le nombre %s de telle sorte que :" % nb)
    exo.append('\\begin{enumerate}')
    cor.append(u"Placer une virgule (en ajoutant éventuellement des zéros) dans\
            le nombre %s de telle sorte que :" % nb)
    cor.append('\\begin{enumerate}')
    for i in range(6):
        dec = [str(nb)[i] for i in range(len(str(nb)))]
        index_dec = random.randrange(6)
        index_valeurs = valeurs_index.pop(random.randrange(len(valeurs_index)))
        exo.append(u"\\item le chiffre %s soit le chiffre des %s : " \
                 % (dec[index_dec], valeurs[index_valeurs]))
        cor.append(u"\\item le chiffre %s soit le chiffre des %s : " \
                 % (dec[index_dec], valeurs[index_valeurs]))
        resultat = ecrit_nombre_decimal(dec, (index_dec + 4) - 
                index_valeurs)
        exo.append('\\dotfill')
        cor.append(Affichage.decimaux(resultat, 0) + '')
    exo.append('\\end{enumerate}')
    cor.append('\\end{enumerate}')


def ecrit_nombre_decimal(dec, index):
    """
    Renvoie une chaine de caractère représentant le nombre dec avec la
    virgule à la place index.
    Ajoute les zéros nécessaires.
    @param dec: décomposition d'un nombre entier
    @param index: place de la virgule dans la liste dec
    """

    if index < 1:
        dec.insert(0, '0')
        dec.insert(1, '.')
        for i in range(-index):
            dec.insert(2, '0')
    elif index < len(dec):
        dec.insert(index, '.')
    else:
        for i in range(index - len(dec)):
            dec.append('0')
    strnb = ""
    for i in range(len(dec)):
        strnb = strnb + dec[i]
    return strnb


def _PlaceVirgule():
    exo = ["\\exercice"]
    cor = ["\\exercice*"]

    tex_place_virgule(exo, cor)
    return (exo, cor)

class PlaceVirgule(LegacyExercise):
    """Placer une virgule"""

    tags = ["Sixième"]
    function = _PlaceVirgule


#===============================================================================
#    Écriture fractionnaire
#===============================================================================


def valeurs_frac():
    n1 = random.randrange(11, 10000)
    p1 = random.randrange(1, 4)
    return (n1, p1)


def choix_trou_frac(exo, cor, n1, p1):
    i = random.randrange(3)
    p2 = random.randrange(2)  # sert à compliquer un peu l'exercice
    if i > 1:
        exo.append('\\item $\\cfrac{%s}{%s}=\\ldots$' % 
                (Affichage.decimaux(n1 * Decimal(10) ** p2),
                    Affichage.decimaux(Decimal(10) ** (p1 + p2))))
        cor.append('\\item $\\cfrac{%s}{%s}=\\mathbf{%s}$' % 
                (Affichage.decimaux(n1 * Decimal(10) ** p2),
                    Affichage.decimaux(Decimal(10) ** (p1 + p2)),
                    Affichage.decimaux(n1 * Decimal(10) ** (-p1), 1)))
    elif i > 0:
        exo.append('\\item $\\cfrac{%s}{\ldots}=%s$' % 
                (Affichage.decimaux(n1 * Decimal(10) ** p2),
        Affichage.decimaux(n1 * Decimal(10) ** (-p1), 1)))
        cor.append('\\item $\\cfrac{%s}{\\mathbf{%s}}=%s$' % 
                (Affichage.decimaux(n1 * Decimal(10) ** p2),
        Affichage.decimaux(Decimal(10) ** (p1 + p2)), \
                Affichage.decimaux(n1 * Decimal(10) ** (-p1), 1)))
    else:
        exo.append('\\item $\\cfrac{\ldots}{%s}=%s$' % 
                (Affichage.decimaux(Decimal(10) ** (p1 + p2)),
        Affichage.decimaux(n1 * Decimal(10) ** (-p1), 1)))
        cor.append('\\item $\\cfrac{\\mathbf{%s}}{%s}=%s$' % 
                (Affichage.decimaux(n1 * Decimal(10) ** p2),
                    Affichage.decimaux(Decimal(10) ** (p1 + p2)),
                    Affichage.decimaux(n1 * Decimal(10) ** (-p1), 1)))

def tex_frac(exo, cor):
    for dummy in range(6):
        (nombre, puissance) = valeurs_frac()
        choix_trou_frac(exo, cor, nombre, puissance)


def _EcritureFractionnaire():
    exo = ["\\exercice", u"Compléter :", '\\begin{multicols}{3}\\noindent',
            '\\begin{enumerate}']
    cor = ["\\exercice*", u"Compléter :", '\\begin{multicols}{3}\\noindent',
            '\\begin{enumerate}']

    tex_frac(exo, cor)

    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    return (exo, cor)

class EcritureFractionnaire(LegacyExercise):
    """Écriture fractionnaire ou décimale"""

    tags = ["Sixième"]
    function = _EcritureFractionnaire


#===============================================================================
#    Décomposition des nombres décimaux
#===============================================================================


def valeurs_dec():
    lpuissances = [3, 2, 1, 0, -1, -2, -3]
    p = []
    v = []
    for dummy in range(3):
        p.append(lpuissances.pop(random.randrange(len(lpuissances))))
        v.append(random.randrange(1, 10))
    return (v, p)


def tex_decomposition(v, p):
    exo, cor = [], []
    for i in range(3):
        if p[i] < 0:
            exo.append('%s\\times \\cfrac{1}{%s}' % (v[i],
                Affichage.decimaux(10 ** (-p[i]), 1)))
            cor.append('%s\\times \\cfrac{1}{%s}' % (v[i],
                Affichage.decimaux(10 ** (-p[i]), 1)))
        else:
            exo.append('%s\\times %s' % (v[i], Affichage.decimaux(10 ** 
                     p[i], 1)))
            cor.append('%s\\times %s' % (v[i], Affichage.decimaux(10 ** 
                     p[i], 1)))
        if i < 2:
            exo.append('+')
            cor.append('+')
        else:
            exo.append('=')
            cor.append('=')
    exo.append('\\dotfill$')
    cor.append('%s$' % Affichage.decimaux(v[0] * 10 ** p[0] + 
             v[1] * 10 ** p[1] + v[2] * 10 ** p[2], 1))
    return " ".join(exo), " ".join(cor)


def tex_dec(exo, cor):
    for dummy in range(6):
        txt = '\\item $'
        (chiffres, puissances) = valeurs_dec()
        txt_exo, txt_cor = tex_decomposition(chiffres, puissances)
        exo.append(txt + txt_exo)
        cor.append(txt + txt_cor)

def _Decomposition():
    exo = ["\\exercice", u"Compléter avec un nombre décimal :",
            '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']
    cor = ["\\exercice*", u"Compléter avec un nombre décimal :",
            '\\begin{multicols}{2}\\noindent', '\\begin{enumerate}']

    tex_dec(exo, cor)

    exo.append('\\end{enumerate}')
    exo.append('\\end{multicols}')
    cor.append('\\end{enumerate}')
    cor.append('\\end{multicols}')
    return (exo, cor)

class Decomposition(LegacyExercise):
    """Décomposition de décimaux"""

    tags = ["Sixième"]
    function = _Decomposition


#===============================================================================
# Classer des nombres dans l'ordre
#===============================================================================


def choix_nombres():
    nb = []
    unite = random.randrange(10)
    for i in range(3):
        n = unite
        for j in range(i + 1):
            n = n + random.randrange(1, 10) * Decimal(10) ** (-(j + 1))
        nb.append(n)
    n = random.randrange(10) + Decimal(random.randrange(10)) / 10
    while n == nb[0]:
        n = random.randrange(10) + Decimal(random.randrange(10)) / 10
    nb.append(n)
    return nb


def classer(exo, cor, ordre):
    lnb = choix_nombres()
    random.shuffle(lnb)
    exo.append("\\item Classer les nombres suivants dans l'ordre %s.\\par    " % 
             ordre)
    cor.append("\\item Classer les nombres suivants dans l'ordre %s.\\par    " % 
             ordre)
    phrase = ""
    for i in range(len(lnb)):
        if i:
            phrase += " \\kern1cm ; \\kern1cm "
        phrase += Affichage.decimaux(lnb[i], 0)
    exo.append(phrase)
    cor.append(phrase + "\\par")
    lnb.sort()
    if ordre == "croissant":
        ordre = "\\textless"
    else:
        ordre = "\\textgreater"
        lnb.reverse()
    phrase = ""
    for i in range(len(lnb)):
        if i:
            phrase += " \\kern1cm %s \\kern1cm " % ordre
        phrase += Affichage.decimaux(lnb[i], 0)
    cor.append(phrase)

def _ClasserNombres():
    exo = ["\\exercice", '\\begin{enumerate}']
    cor = ["\\exercice*", '\\begin{enumerate}']
    classer(exo, cor, "croissant")
    classer(exo, cor, "décroissant")
    exo.append('\\end{enumerate}')
    cor.append('\\end{enumerate}')
    return (exo, cor)

class ClasserNombres(LegacyExercise):
    """Classer des nombres décimaux"""

    tags = ["Sixième"]
    function = _ClasserNombres



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#

import random
from pyromaths.ex import LegacyExercise

#
# ------------------- PGCD -------------------


def tex_trouve_diviseur(a):  # trouve si les nombres dans le tuple a sont divisible par 10, 2, 5, 9 ou 3 (dans cet ordre)
    if a[0] % 10 == 0 and a[1] % 10 == 0:
        return '\\nombre{' + str(a[0]) + '} et \\nombre{' + str(a[1]) + \
            u'} se terminent tous les deux par zéro donc ils sont divisibles par 10.\\par\n' + \
            '\\nombre{' + str(a[0]) + '} et \\nombre{' + str(a[1]) + \
            '} ne sont donc pas premiers entre eux'
    elif a[0] % 2 == 0 and a[1] % 2 == 0:
        return '\\nombre{' + str(a[0]) + '} et \\nombre{' + str(a[1]) + \
            '} sont deux nombres pairs donc ils sont divisibles par 2.\\par\n' + \
            '\\nombre{' + str(a[0]) + '} et \\nombre{' + str(a[1]) + \
            '} ne sont donc pas premiers entre eux'
    elif a[0] % 5 == 0 and a[1] % 5 == 0:
        return '\\nombre{' + str(a[0]) + '} et \\nombre{' + str(a[1]) + \
            u'} se terminent tous les deux par zéro ou cinq donc ils sont divisibles par 5.\\par\n' + \
            '\\nombre{' + str(a[0]) + '} et \\nombre{' + str(a[1]) + \
            '} ne sont donc pas premiers entre eux'
    elif a[0] % 9 == 0 and a[1] % 9 == 0:
        return 'La somme des chiffres de \\nombre{' + str(a[0]) + \
            '} et celle de \\nombre{' + str(a[1]) + \
            '} sont divisibles par neuf donc ils sont divisibles par 9.\\par\n' + \
            '\\nombre{' + str(a[0]) + '} et \\nombre{' + str(a[1]) + \
            '} ne sont donc pas premiers entre eux'
    elif a[0] % 3 == 0 and a[1] % 3 == 0:
        return 'La somme des chiffres de \\nombre{' + str(a[0]) + \
            '} et celle de \\nombre{' + str(a[1]) + \
            '} sont divisibles par trois donc ils sont divisibles par 3.\\par\n' + \
            '\\nombre{' + str(a[0]) + '} et \\nombre{' + str(a[1]) + \
            '} ne sont donc pas premiers entre eux'


def valeurs_pgcd():  # creer un tuple contenant les deux nombres dont on cherche le pgcd
    reste = random.choice((2, 3, 5, 9, 10)) * random.choice((7, 11, 13,
            17, 19, 23, 31))
    diviseur = reste * random.randrange(1, 10)
    for dummy in range(random.randrange(2, 5)):
        (diviseur, reste) = (diviseur * random.randrange(1, 10) + reste,
                             diviseur)
    return (diviseur, reste)


def algo_euclide(a):  # renvoi une liste contenant (dividende,diviseur,quotient,reste) pour chaque etape
    liste = []
    if a[0] > a[1]:
        (dividende, diviseur) = (a[0], a[1])
    else:
        (dividende, diviseur) = (a[1], a[0])
    (quotient, reste) = (dividende // diviseur, dividende % diviseur)
    liste.append((dividende, diviseur, quotient, reste))
    while reste != 0:
        (dividende, diviseur) = (diviseur, reste)
        (quotient, reste) = (dividende // diviseur, dividende % diviseur)
        liste.append((dividende, diviseur, quotient, reste))
    return liste


def simplifie_fraction_pgcd(l):  # renvoie le nombre par lequel on peut simplifier la fraction et la fraction simplifiée
    (pgcd, n0, d0) = (l[len(l) - 1][1], l[0][0], l[0][1])
    (n1, d1) = (l[0][0] // pgcd, l[0][1] // pgcd)
    return (n0, d0, n0, pgcd, d0, pgcd, n1, d1)


def tex_algo_euclide(l):  # renvoie l'ecriture au format tex de l'algorithme d'Euclide
    lignes = []
    for i in range(len(l)):
        lignes.append('\\nombre{%s}=\\nombre{%s}\\times\\nombre{%s}+\\nombre{%s}' % 
                      l[i])
    lignes.append('\\fbox{Donc le \\textsc{pgcd} de \\nombre{%s} et \\nombre{%s} est %s}.\n' % 
                  (l[0][0], l[0][1], l[len(l) - 1][1]))
    return lignes


def tex_simplifie_fraction_pgcd(a):  # renvoie l'ecriture au format tex de la simplification de la fraction
    return '''\\begin{align*}
\\cfrac{\\nombre{%s}}{\\nombre{%s}} &= \\cfrac{\\nombre{%s}\\div%s}{\\nombre{%s}\\div%s}\\\\\n      &= \\boxed{\\cfrac{\\nombre{%s}}{\\nombre{%s}}}
\\end{align*}''' % a

def _tex_pgcd():
    nombres = valeurs_pgcd()
    exo = ['\\exercice']
    exo.append('\\begin{enumerate}')
    exo.append('\\item Les nombres \\nombre{%s} et \\nombre{%s} sont-ils premiers entre eux ?  ' % nombres)
    exo.append('\\item Calculer le plus grand commun diviseur (\\textsc{pgcd}) de \\nombre{%s} et \\nombre{%s}.' % nombres)
    exo.append(u'\\item Simplifier la fraction $\\cfrac{\\nombre{%s}}{\\nombre{%s}}$ pour la rendre irréductible en indiquant la méthode.\n' % nombres)
    exo.append('\\end{enumerate}')
    cor = ['\\exercice*']
    cor.append('\\begin{enumerate}')
    cor.append('\\item Les nombres \\nombre{%s} et \\nombre{%s} sont-ils premiers entre eux ?\\par ' % nombres)
    cor.append(tex_trouve_diviseur(nombres))
    cor.append('\\item Calculer le plus grand commun diviseur (\\textsc{pgcd}) de \\nombre{%s} et \\nombre{%s}.\\par' % nombres)
    cor.append('On calcule le \\textsc{pgcd} des nombres \\nombre{%s} et \\nombre{%s} en utilisant l\'algorithme d\'Euclide.' % nombres)
    l = algo_euclide(nombres)
    tex_liste = tex_algo_euclide(l)
    for i in range(len(l)):
        cor.append(u'\\[ %s' % tex_liste[i] + '\\] ')
    cor.append(tex_liste[len(l)])
    cor.append(u'\\item Simplifier la fraction $\\cfrac{\\nombre{%s}}{\\nombre{%s}}$ pour la rendre irréductible en indiquant la méthode.' % nombres)
    cor.append(tex_simplifie_fraction_pgcd(simplifie_fraction_pgcd(l)))
    cor.append('\\end{enumerate}')
    return (exo, cor)

class tex_pgcd(LegacyExercise):
    """PGCD"""

    tags = ["Troisième"]
    function = _tex_pgcd



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2014 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
from random import randrange, shuffle
from pyromaths.outils import Priorites3
from pyromaths.classes.PolynomesCollege import factoriser
from pyromaths.ex import LegacyExercise

def _factorisation():
    """Génère un exercice de factorisation utilisant les identités remarquables ou
    la distributivité
    """
    l = [randrange(1, 11) for dummy in range(21)]
    diff = [True, False, False]
    shuffle(diff)
    exo = [id_rem1, id_rem2]
    lexo = [exo[randrange(2)](l[0], l[1])]
    lexo.append(id_rem3(l[2], l[3]))
    lexo.append(id_rem3bis(l[4], l[5], l[6]))
    lexo.append(facteur_commun1(l[7:13], diff=diff.pop()))
    shuffle(lexo)
    exo = [facteur_commun2, facteur_commun3]
    shuffle(exo)
    lexo.append(exo[0](l[13:17], diff=diff.pop()))
    lexo.append(exo[1](l[17:21], diff=diff.pop()))

    exo = ["\\exercice", u"Factoriser chacune des expressions littérales suivantes :"]
    exo.append("\\begin{multicols}{2}")
    cor = ["\\exercice*", u"Factoriser chacune des expressions littérales suivantes :"]
    cor.append("\\begin{multicols}{2}")
    for i in range(len(lexo)):
        p = [lexo[i]]
        while True:
            fact = factoriser(p[-1])
            if fact:
                p.append(fact)
            else: break
        p = Priorites3.texify([Priorites3.splitting(p[j]) for j in range(len(p))])
        cor.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), p[j]) for j in range(len(p) - 1)]))
        cor.append('\\\\')
        cor.append('\\fbox{$%s=%s$}\\\\\n' % (chr(i + 65), p[-1]))
    exo.append('\\\\\n'.join(['$%s=%s$' % (chr(i + 65), Priorites3.texify([Priorites3.splitting(lexo[i])])[0]) for i in range(len(lexo))]))
    exo.append("\\end{multicols}")
    cor.append("\\end{multicols}")
    return exo, cor

def id_rem1(a, b, details=1):
    """Construit un Polynome de la forme ax^2+2abx+b^2
    Renvoie une chaine"""
    return 'Polynome([[%r, 2], [%r, 1], [%r, 0]], details=%s)' % (a ** 2, 2 * a * b, b ** 2, details)
def id_rem2(a, b, details=1):
    """Construit un Polynome de la forme ax^2-2abx+b^2
    Renvoie une chaine"""
    return 'Polynome([[%r, 2], [%r, 1], [%r, 0]], details=%s)' % (a ** 2, -2 * a * b, b ** 2, details)
def id_rem3(a, b, details=1):
    """Construit un Polynome de la forme a^2x^2-b^2
    Renvoie une chaine"""
    sgn = randrange(2)
    return 'Polynome([[%r, 2], [%r, 0]], details=%s)' % (a ** 2 * (-1) ** sgn, b ** 2 * (-1) ** (sgn + 1), details)
def id_rem3bis(a, b, c, details=1):
    """Construit un Polynome de la forme (ax+b)^2-c^2
    Renvoie une chaine"""
    sgn = randrange(2)
    lpolynomes = ['Polynome([[%r, 1], [%r, 0]], details=%s)**2' % (a * (-1) ** randrange(3), b * (-1) ** randrange(3), details)]
    lpolynomes.append('Polynome([[%r, %r]], details=%s)' % (c ** 2, 2 * randrange(2), details))
    shuffle(lpolynomes)
    lop = ['+', '-']
    shuffle(lop)
    poly = ''
    for i in range(2):
        if i != 0 or lop[0] == '-': poly += lop[i]
        poly += lpolynomes[i]
    return poly
def facteur_commun1(lcoeff, details=1, diff=False):
    """Construit une somme de la forme (ax+b)(cx+d)+(ax+b)(ex+f)
    Si diff, alors (ax+b)(cx+d)-(ax+b)(ex+f)
    lcoeff est une liste de décimaux de longueur 6
    Renvoie une chaine"""
    lpolynomes = ['Polynome([[%r, 1], [%r, 0]], details=%s)' % (lcoeff[2 * i] * (-1) ** randrange(3), lcoeff[2 * i + 1] * (-1) ** randrange(3), details) for i in range(3)]
    lpolynomes.insert(randrange(2, 4), lpolynomes[randrange(2)])
    lop = [0, '*', '+', '*']
    if diff: lop[randrange(2) * 2] = '-'
    poly = ''
    for i in range(4):
        if lop[i]: poly += lop[i]
        poly += lpolynomes[i]
    return poly
def facteur_commun2(lcoeff, details=1, diff=False):
    """Construit une somme de la forme (ax+b)(cx+d)+(ax+b)
    si diff, (ax+b)(cx+d)-(ax+b) ou -(ax+b)(cx+d)+(ax+b)
    lcoeff est une liste de décimaux de longueur 4
    Renvoie une chaine"""
    lpolynomes = ['Polynome([[%r, 1], [%r, 0]], details=%s)' % (lcoeff[2 * i], lcoeff[2 * i + 1] * (-1) ** randrange(3), details) for i in range(2)]
    a = lpolynomes[0]
    shuffle(lpolynomes)
    lop = [ '*', '+']
    if diff: lop[1] = '+-'[randrange(2)]
    shuffle(lop)
    if lop[0] == '*':
        lpolynomes.append(a)
    else:
        lpolynomes.insert(0, a)
    if diff and lop.count('-') == 0: lop.insert(0, '-')
    else: lop.insert(0, 0)
    poly = ''
    for i in range(3):
        if lop[i]: poly += lop[i]
        poly += lpolynomes[i]
    return poly
def facteur_commun3(lcoeff, details=1, diff=False):
    """Construit une somme de la forme (ax+b)(cx+d)+(ax+b)**2
    lcoeff est une liste de décimaux de longueur 4
    Renvoie une chaine"""
    lpolynomes = ['Polynome([[%r, 1], [%r, 0]], details=%s)' % (lcoeff[2 * i] * (-1) ** randrange(3), lcoeff[2 * i + 1] * (-1) ** randrange(3), details) for i in range(2)]
    a = lpolynomes[0]
    shuffle(lpolynomes)
    lop = [ '*', '+']
    if diff: lop[1] = '+-'[randrange(2)]
    shuffle(lop)
    if lop[0] == '*':
        lpolynomes.append(a + '**2')
    else:
        lpolynomes.insert(0, a + '**2')
    if diff and lop.count('-') == 0: lop.insert(0, '-')
    else: lop.insert(0, 0)
    poly = ''
    for i in range(3):
        if lop[i]: poly += lop[i]
        poly += lpolynomes[i]
    return poly

class factorisation(LegacyExercise):
    """Factorisations"""

    tags = ["Troisième"]
    function = _factorisation



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

import random

from pyromaths.outils import Affichage
from pyromaths.ex import LegacyExercise

def boxes():
    """Crée les boites pour insérer les figures dans un environnement 36x16"""
    b0x, b0y = random.randrange(5, 2 * 36 // 3 - 1), random.randrange(5, 2 * 16 // 3 + 1)
    b1x, b1y = random.randrange(5, 2 * (36 - b0x) // 3 - 1), random.randrange(5, 2 * 16 // 3 + 1)
    b2x, b2y = 36 - b0x - b1x - 2, random.randrange(5, int(2 * 16 // 3 + 1))
    b3x, b3y = b0x, 16 - b0y - 1
    b4x, b4y = b1x, 16 - b1y - 1
    b5x, b5y = b2x, 16 - b2y - 1
    return (b0x, b0y), (b1x, b1y), (b2x, b2y), (b3x, b3y), (b4x, b4y), (b5x,
                                                                        b5y)


def carre(dim, n_fig):
    """Dessine en psTricks un carre de dimensions inf(dim) et numérote la figure
    avec n_fig"""
    if dim[0] < dim[1]:
        dim = (dim[0], dim[0])
    else:
        dim = (dim[1], dim[1])
    f = "\\psframe[fillstyle=hlines](0, 0)(%s, %s)\n" % dim
    f += "\\rput(%.2f,%.2f)" % isobarycentre((0, 0), dim)
    f += "{\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{figure %s}} " % n_fig
    s = u"Aire de la figure %s : $%s = %s$~unités d'aire" % (n_fig,
                                                             aire_rectangle(dim)[0], aire_rectangle(dim)[1])
    return f, f, s


def rectangle(dim, n_fig):
    """Dessine en psTricks un rectangle de dimensions dim et numérote la figure
    avec n_fig"""
    f = "\\psframe[fillstyle=hlines](0, 0)(%s, %s)\n" % dim
    f += "\\rput(%.2f,%.2f)" % isobarycentre((0, 0), dim)
    f += "{\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{figure %s}}" % n_fig
    s = u"Aire de la figure %s : $%s = %s$~unités d'aire" % (n_fig,
                                                             aire_rectangle(dim)[0], aire_rectangle(dim)[1])
    return f, f, s


def parallelogramme(dim, n_fig):
    """Dessine en psTricks un parallelogramme inclus dans un rectangle de
    dimensions dim et numérote la figure avec n_fig"""
    base_h = random.randrange(2)
    if base_h:
        tab = random.randrange(1, min(dim[0] // 2, dim[0] - 3)) * (-1) ** random.randrange(2)
        if tab > 0:
            s0, s1, s2, s3 = (tab, 0), (dim[0], 0), (dim[0] - tab, dim[1]), (0,
                                                                             dim[1])
            frame0, frame1 = s0, dim
        else:
            s0, s1, s2, s3 = (0, 0), (dim[0] + tab, 0), (dim[0], dim[1]), (-tab,
                                                                           dim[1])
            frame0, frame1 = s0, (dim[0] + tab, dim[1])
    else:
        tab = random.randrange(1, min(dim[1] // 2, dim[1] - 3)) * (-1) ** random.randrange(2)
        if tab > 0:
            s0, s1, s2, s3 = (0, tab), (dim[0], 0), (dim[0], dim[1] - tab), (0,
                                                                             dim[1])
            frame0, frame1 = s0, dim
        else:
            s0, s1, s2, s3 = (0, 0), (dim[0], -tab), (dim[0], dim[1]), (0,
                                                                        dim[1] + tab)
            frame0, frame1 = s0, (dim[0], dim[1] + tab)
    f = "\\pspolygon[fillstyle=hlines]%s%s%s%s\n" % (s0,
                                                     s1, s2, s3)
    f += "\\rput(%.2f,%.2f)" % isobarycentre((0, 0), dim)
    f += "{\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{figure %s}} " % n_fig
    fc = "\\pspolygon%s%s%s%s\n" % (s0, s1, s2, s3)
    fc += "\\psframe[linestyle=dashed, fillstyle=hlines]%s%s\n" % (frame0, frame1)
    fc += "\\rput(%.2f,%.2f)" % isobarycentre((0, 0), dim)
    fc += "{\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{figure %s}} " % n_fig
    s = "Aire de la figure %s : " % n_fig
    s += u"c'est l'aire du rectangle en pointillés.\\par\n"
    s += u"$%s = %s$~unités d'aire" % (aire_rectangle(frame0, frame1)[0],
                                       aire_rectangle(frame0, frame1)[1])
    return f, fc, s


def triangle_rectangle(dim, n_fig):
    """Dessine en psTricks un triangle rectangle dans une boite de dimensions
    dim et numérote la figure avec n_fig"""
    sommets = [(0, 0), (dim[0], 0), dim, (0, dim[1])]
    s0 = random.randrange(4)
    s1, s2 = (s0 + 1) % 4, (s0 + 2) % 4
    s0, s1, s2 = sommets[s0], sommets[s1], sommets[s2]
    f = "\\pspolygon[fillstyle=hlines]%s%s%s\n" % (s0, s1,
                                                   s2)
    f += "\\rput(%.2f,%.2f)" % isobarycentre(s0, s1, s2)
    f += "{\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{figure %s}} " % n_fig
    fc = "\\psframe[linestyle=dashed]%s%s\n" % (s0, s2)
    fc += f
    s = "Aire de la figure %s : " % n_fig
    s += u"c'est la moitié de l'aire du rectangle en pointillés.\\par\n"
    s += u"$(%s) \\div 2= %s$~unités d'aire" % (aire_rectangle(dim)[0],
                                                Affichage.decimaux(aire_rectangle(dim)[1] / 2, 1))
    return f, fc, s


def triangle_base(dim, n_fig):
    """Dessine en psTricks un triangle avec une base horizontale ou verticale
    dans une boite de dimensions dim et numérote la figure avec n_fig"""
    sommets = [(0, 0), (dim[0], 0), dim, (0, dim[1])]
    s0 = random.randrange(4)
    s1 = (s0 + 1) % 4
    x0, x1 = sommets[(s0 + 2) % 4][0], sommets[(s0 + 3) % 4][0]
    y0, y1 = sommets[(s0 + 2) % 4][1], sommets[(s0 + 3) % 4][1]
    if x0 > x1: x0, x1 = x1, x0
    if y0 > y1: y0, y1 = y1, y0
    if x0 != x1: x0 = random.randrange(x0 + 1, x1)
    if y0 != y1: y0 = random.randrange(y0 + 1, y1)
    s2 = (x0, y0)
    s0, s1 = sommets[s0], sommets[s1]
    f = "\\pspolygon[fillstyle=hlines]%s%s%s\n" % (s0, s1,
                                                   s2)
    f += "\\rput(%.2f,%.2f)" % isobarycentre(s0, s1, s2)
    f += "{\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{figure %s}} " % n_fig
    fc = "\\psframe[linestyle=dashed]%s%s\n" % (s0, s2)
    fc += "\\psframe[linestyle=dashed]%s%s\n" % (s2, s1)
    fc += f
    s = "Aire de la figure %s : " % n_fig
    s += u"c'est la moitié de l'aire du rectangle en pointillés.\\par\n"
    s += u"$(%s) \\div 2= %s$~unités d'aire" % (aire_rectangle(dim)[0],
                                                Affichage.decimaux(aire_rectangle(dim)[1] / 2, 1))
    return f, fc, s


def triangle_qcq(dim, n_fig):
    """Dessine en psTricks un triangle quelconque dans une boite de dimensions
    dim et numérote la figure avec n_fig"""
    sommets = [(0, 0), (dim[0], 0), dim, (0, dim[1])]
    s0 = random.randrange(4)
    angle0, angle1, angle2 = sommets[(s0 + 1) % 4], sommets[(s0 + 2) % 4], sommets[(s0 + 3) % 4]
    x0, x1 = sommets[(s0 + 1) % 4][0], sommets[(s0 + 2) % 4][0]
    y0, y1 = sommets[(s0 + 1) % 4][1], sommets[(s0 + 2) % 4][1]
    if x0 > x1: x0, x1 = x1, x0
    if y0 > y1: y0, y1 = y1, y0
    if x0 != x1: x0 = random.randrange(x0 + 1, x1)
    if y0 != y1: y0 = random.randrange(y0 + 1, y1)
    s1 = (x0, y0)
    x0, x1 = sommets[(s0 + 2) % 4][0], sommets[(s0 + 3) % 4][0]
    y0, y1 = sommets[(s0 + 2) % 4][1], sommets[(s0 + 3) % 4][1]
    if x0 > x1: x0, x1 = x1, x0
    if y0 > y1: y0, y1 = y1, y0
    if x0 != x1: x0 = random.randrange(x0 + 1, x1)
    if y0 != y1: y0 = random.randrange(y0 + 1, y1)
    s2 = (x0, y0)
    s0 = sommets[s0]
    f = "\\pspolygon[fillstyle=hlines]%s%s%s\n" % (s0, s1,
                                                   s2)
    f += "\\rput(%.2f,%.2f)" % isobarycentre(s0, s1, s2)
    f += "{\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{figure %s}}" % n_fig
    fc = "\\psframe[linestyle=dashed](0,0)(%s,%s) " % dim
    fc += "\\rput(%.2f,%.2f){\\pscirclebox{1}} " % isobarycentre(s0, s1, angle0)
    fc += "\\rput(%.2f,%.2f){\\pscirclebox{2}} " % isobarycentre(s1, s2, angle1)
    fc += "\\rput(%.2f,%.2f){\\pscirclebox{3}}\n" % isobarycentre(s2, s0, angle2)
    fc += f
    s = "Aire de la figure %s : " % n_fig
    s += u"on calcule l'aire du rectangle en pointillés et on soustrait "
    s += "les aires des triangles rectangles \\pscirclebox{1}, "
    s += "\\pscirclebox{2} et \\pscirclebox{3}.\\par\n"
    s += "$(%s) - (%s) \\div 2 - (%s) \\div 2 - (%s) \\div 2 " % \
         (aire_rectangle(dim)[0], aire_rectangle(s0, s1)[0],
          aire_rectangle(s1, s2)[0], aire_rectangle(s2, s0)[0])
    s += u"= %s$~unités d'aire" % Affichage.decimaux(
        aire_rectangle(dim)[1] - aire_rectangle(s0, s1)[1] / 2 - aire_rectangle(s0, s2)[1] / 2 - aire_rectangle(s1, s2)[
            1] / 2, 1)
    return f, fc, s


def aire_rectangle(pos1, pos2=(0, 0)):
    """Affiche le calcul et le résultat de l'aire d'un rectangle, les
    coordonnées de deux sommets opposés étant pos1 et pos2"""
    dx = abs(pos1[0] - pos2[0])
    dy = abs(pos1[1] - pos2[1])
    return "%s \\times %s" % (dx, dy), dx * dy


def isobarycentre(*args):
    nbarg = len(args) * 1.
    isobar = [0, 0]
    for i in args:
        isobar = [isobar[0] + i[0] // nbarg, isobar[1] + i[1] // nbarg]
    return tuple(isobar)


def figure():
    """Dessine en psTricks la figure de l'exercice sur les aires (quadrillage et
    figures géométriques)"""
    exo, cor, sol = [], [], ["\\begin{enumerate}"]
    t = "\\begin{pspicture}(0,0)(18,9)\n"
    t += "\\psgrid[subgriddiv=2, gridlabels=0pt]\n"
    t += "\\psframe[fillstyle=vlines, hatchsep=1pt](0,0)(.5,.5)\n"
    t += u"\\rput[l](0.6,0.25){\\psframebox[linecolor=white, fillcolor=white, fillstyle=solid]{unité d'aire}}\n"
    t += "\\psset{unit=5mm}\n"
    exo.append(t)
    cor.append(t)
    dim0, dim1, dim2, dim3, dim4, dim5 = boxes()
    xtab0 = 0
    ytab3 = ytab4 = ytab5 = 2
    xtab1 = dim0[0] + 1
    xtab2 = dim0[0] + dim1[0] + 2
    ytab0 = dim3[1] + 3
    ytab1 = dim4[1] + 3
    ytab2 = dim5[1] + 3
    fig = [carre, rectangle, triangle_rectangle, triangle_base, triangle_qcq, parallelogramme]
    for i in range(len(fig)):
        r = fig.pop(random.randrange(len(fig)))(eval('dim' + str(i)), i + 1)
        exo.append("\\rput(%s,%s){\n" % (eval('xtab' + str(i % 3)),
                                         eval('ytab' + str(i))) + r[0] + '\n}')
        cor.append("\\rput(%s,%s){\n" % (eval('xtab' + str(i % 3)),
                                         eval('ytab' + str(i))) + r[1] + '\n}')
        sol.append("\\item %s" % r[2])
    exo.append("\\end{pspicture}")
    cor.append("\\end{pspicture}")
    cor.extend(sol)
    cor.append("\\end{enumerate}")
    return "\n".join(exo), "\n".join(cor)


def _aires():
    exo = ["\\exercice", u"Calculer l'aire de chacune des figures suivantes dans l'unité d'aire donnée :\\par"]
    cor = ["\\exercice*", u"Calculer l'aire de chacune des figures suivantes dans l'unité d'aire donnée :\\par"]
    exercice = figure()
    exo.append(exercice[0])
    cor.append(exercice[1])
    return exo, cor


class aires(LegacyExercise):
    """Aires et quadrillage"""

    tags = ["Sixième"]
    function = _aires



#!/usr/bin/env python3

# Pyromaths
# Un programme en Python qui permet de créer des fiches d'exercices types de
# mathématiques niveau collège ainsi que leur corrigé en LaTeX.
# Copyright (C) 2006 -- Jérôme Ortais (jerome.ortais@pyromaths.org)
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
#
#----------------------------------------------------------------------
# Pyromaths : GESTION DES PRIORITES
#----------------------------------------------------------------------

# from Affichage import decimaux
from pyromaths.outils import Arithmetique


def cherche_classe(calcul, index):
    """**cherche_classe**\ (*calcul*\ , *index*)

    Recherche le premier objet polynôme ou fraction dans la chaîne calcul à une
    position supérieure ou égale à index, sans expression régulière

    :param calcul: le calcul à tester
    :type calcul: string
    :param index: position à partir de laquelle effectuer le test
    :type index: integer

    >>> from pyromaths.classes.PolynomesCollege import Polynome
    >>> from pyromaths.classes.Fractions import Fraction
    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.cherche_classe('Polynome([[-4, 2]], "x")-Polynome([[-5, 1], [4, 2], [1, 0]], "x")+Polynome([[6,  0], [5, 1]], "x")', 5)
    Polynome([[-5, 1], [4, 2], [1, 0]], "x")
    >>> Priorites3.cherche_classe('Polynome([[-4, 2]], "x")+Fraction(3,2)+Polynome([[6,  0], [5, 1]], "x")', 5)
    Fraction(3,2)

    :rtype: string
    """
    classes = ["Polynome(", "Fraction(", "SquareRoot("]
    indexes = [calcul.find(classe, index) for classe in classes]
    position, i = 0, 1
    while i < len(classes):
        if 0 <= indexes[i] and (indexes[position] < 0 or indexes[i] < indexes[position]):
            position = i
        i += 1
    if indexes[position] < 0: return None
    classe = classes[position]
    index = indexes[position]
    par, i = 1, index + len(classe)  # nombre de parenthèses ouvertes, début de recherche
    while par:
        if calcul.find("(", i) > 0:
            i = min(calcul.find("(", i), calcul.find(")", i))
        else:
            i = calcul.find(")", i)
        if calcul[i] == ")": par += -1
        else: par += 1
        i += 1
    return calcul[index:i]

def cherche_decimal(calcul, index):
    """**cherche_decimal**\ (*calcul*\ , *index*)

    Recherche le premier nombre décimal dans la chaîne calcul à une position
    supérieure ou égale à index, sans expression régulière.
    Un nombre décimal peut être entouré de parenthèses s'il a un signe, ou avoir
    un signe s'il est en début de chaîne.

    Cette fonction est 130 fois plus rapide qu'avec une expression régulière.

    :param calcul: le calcul à tester
    :type calcul: string
    :param index: position à partir de laquelle effectuer le test
    :type index: integer

    >>> from pyromaths.outils import Priorites3
    >>> p = '-Polynome([[-4, 2]], "x")*6**2+3'
    >>> Priorites3.cherche_decimal(p, 1)
    4
    >>> p = '-6*(-11)*(-5)'
    >>> Priorites3.cherche_decimal(p, 1)
    6
    >>> Priorites3.cherche_decimal(p, 0)
    6

    **TODO :** vérifier ces deux derniers exemples. Je pense que je devrais récupérer -6

    :rtype: string
    """
    end = len(calcul)
    while index <= end:
        try:
            int(calcul[index])
        except:
            index += 1
        else:
            break
    if index > end: return None
    while end > index:
        try:
            float(calcul[index:end])
        except ValueError:
            end -= 1
        else:
            break
    if index > 1 and (calcul[index - 2:index] == "(+" or calcul[index - 2:index] ==
            "(-") :
        if end < len(calcul) and calcul[end] == ")":
            index -= 2
            end += 1
        else:
            index -= 1
    return calcul[index:end]

def cherche_string(calcul, index):
    apostrophes = ["'", "\""]
    indexes = [calcul.find(apostrophe, index) for apostrophe in apostrophes]
    position, i = 0, 1
    while i < len(apostrophes):
        if 0 <= indexes[i] and (indexes[position] < 0 or indexes[i] < indexes[position]):
            position = i
        i += 1
    if indexes[position] < 0: return None
    apostrophe = apostrophes[position]
    index = indexes[position] + 1
    i = calcul.find(apostrophe, index)

    return calcul[index:i]

def cherche_operateur(calcul, index):
    """**cherche_operateur**\ (*calcul*\ , *min_i*)

    Recherche le premier opérateur +, -, \*, /, ^ dans la chaîne calcul à une position
    supérieure ou égale à min_i, sans expression régulière.

    :param calcul: le calcul à tester
    :type calcul: string
    :param min_i: position à partir de laquelle effectuer le test
    :type min_i: integer

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.cherche_operateur('-Polynome([[-4, 2]], "x")*6**2+3', 1)
    -
    >>> p = '-6*(-11)*(-5)'
    >>> Priorites3.cherche_operateur(p, 1)
    *
    >>> Priorites3.cherche_operateur(p, 0)
    -

    :rtype: string
    """
    operateur = ['**', '+', '-', '*', '/', '^']
    l = []
    for op in operateur:
        try:
            l.append(calcul[index:].index(op) + index)
        except:
              l.append(None)
    min_i = 0
    for i in range (1, len(l)):
        if l[i] is not None and (l[min_i] is None or l[i] < l[min_i]):
            min_i = i
    if l[min_i] != None:
        return operateur[min_i]
    else:
        return None

def split_calcul(calcul):
    """**split_calcul**\ (*calcul*)

    Partitionne la chaîne de caractères pour obtenir une liste d'opérateurs,
    de parenthèses, de polynômes et de nombres

    :param calcul: le calcul à traiter
    :type calcul: string

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.split_calcul('-Polynome([[-4, 2]], "x")*6**2+3')
    ['-', 'Polynome([[-4, 2]], "x")', '*', '6', '**', '2', '+', '3']
    >>> Priorites3.split_calcul('-6*(-11)*(-5)')
    ['-', '6', '*', '(-11)', '*', '(-5)']
    >>> Priorites3.split_calcul("Polynome('Fraction(1,3)x')+Polynome('x')")
    ["Polynome('Fraction(1,3)x')", '+', "Polynome('x')"]

    **TODO :** vérifier ce dernier exemple. Je pense que je devrais récupérer -6

    :rtype: list
    """
    l = []
    findings = (cherche_classe, cherche_string, cherche_decimal)  # , cherche_operateur)
    for finding in findings:
        nb = finding(calcul, 0)
        while nb:
            l.extend(calcul.partition(nb))
            l[-3] = l[-3].strip()
            l[-2] = l[-2].strip()
            l[-1] = l[-1].strip()
            if l[-3] and l[-2]:
                l0 = split_calcul(l[-3])
                if l0:
                    l0.extend(l[-2:])
                    l = l[:-3]
                    l.extend(l0)
            elif not l[-3]:
                l.pop(-3)
            calcul = l[-1]
            nb = finding(calcul, 0)
            l = l[:-1]
    if calcul: l.append(calcul)
    while l and not l[0]:
        l.pop(0)
    return l

def EstNombre(value):
    """**EstNombre**\ (*value*)

    Teste si `value` est une valeur, c'est à dire un entier, un réel, un polynôme, une fraction, une racine carrée.
    Attention : (-11) est considéré comme un nombre

    :param value: la valeur à traiter
    :type value: string

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.EstNombre('-15')
    True
    >>> Priorites3.EstNombre('Polynome([[-4, 2]], "x")')
    True

    :rtype: list
    """
    from pyromaths.classes.PolynomesCollege import Polynome
    from pyromaths.classes.Fractions import Fraction
    from pyromaths.classes.SquareRoot import SquareRoot
    classe = cherche_classe(value, 0)
    if  classe:
        if classe != value: return False
        else: return True
    #===========================================================================
    # if cherche_operateur(value, 1):
    #     return False
    #===========================================================================
    try:
        return isinstance(eval(value), (float, int, Polynome, Fraction, SquareRoot))
    except:
        return False


def splitting(calcul):
    """**splitting**\ (*calcul*)

    Partitionne la chaîne de caractères pour obtenir une liste d'opérateurs,
    de parenthèses, de polynômes et de nombres puis arrange la liste des
    opérandes et opérateurs

    :param calcul: le calcul à traiter
    :type calcul: string

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.splitting('-Polynome([[-4, 2]], "x")*6**2+3')
    ['-', 'Polynome([[-4, 2]], "x")', '*', '6', '**', '2', '+', '3']
    >>> Priorites3.splitting('-6*(-11)*(-5)')
    ['-6', '*', '(-11)', '*', '(-5)']
    >>> Priorites3.splitting("Fraction(1,7)x^2-Fraction(3,8)x-1")
    ['Fraction(1,7)', 'x', '^', '2', '-', 'Fraction(3,8)', 'x', '-', '1']
    >>> Priorites3.splitting('-7**2')
    ['-', '7', '**', '2']

    :rtype: list
    """
    if calcul == "": return []
    l = split_calcul(calcul)
    if l[0] == "+" and len(l) > 1: l[0] += l.pop(1)
    elif l[0] == "-" and len(l) > 1 and l[1][0] in "0123456789":
        if len(l)==2 or (len(l)>2 and l[2]!="**" and l[2]!="^"): l[0] += l.pop(1)
    j = 0
    while j < len(l):
        op, i, t = l[j], 0, []
        if op == "**": j += 1
        else:
            if not EstNombre(op):
                while i < len(op):
                    if op[i:i + 2] == "**":
                        t.append("**")
                        i += 2
                    else:
                        t.append(op[i])
                        i += 1
                l[j:j + 1] = t
                j += len(t)
            else: j += 1
    # Recherche les erreurs d'écriture du style 3+-5 à la place de 3+(-5)
    index = 0
    while l[index:].count('-'):
        i = l.index('-', index)
        if i and l[i - 1][-1] in '+-*/^':
            del l[i]
            if l[i][0] == '-': l[i] == l[i][1:]
            else:l[i] = '(-%s)' % l[i]
        index = i + 1
    for dummy in range(l.count('\'')):
        l.remove('\'')
    for dummy in range(l.count('"')):
        l.remove('"')
    return l

def recherche_parentheses(calcul):
    """**recherche_parentheses**\ (*calcul*)

    Recherche les premières parenthèses (éventuellement intérieures) dans une expression

    :param calcul: le calcul à traiter
    :type calcul: list

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.recherche_parentheses(['-6', '*', '(-11)', '*', '(-5)'])
    None
    >>> Priorites3.recherche_parentheses(['-9', '-', '6', '*', '(', '(-2)', '-', '4', ')'])
    (4, 9)

    :rtype: tuple
    """
    if calcul.count("("):
        debut = calcul.index("(")
        if calcul[debut + 1:].count(")"): fin = calcul[debut:].index(")") + debut
        else: return None
        for dummy in range(calcul[debut + 1:fin].count("(")):
            debut += calcul[debut + 1:].index("(") + 1
        return (debut, fin + 1)
    else:
        return None

def recherche_puissance(calcul):
    """**recherche_puissance**\ (*calcul*)

    Recherche la première occurrence d'une puissance dans une liste de
    calculs. Elle doit être entourée de deux nombres.

    :param calcul: le calcul à traiter
    :type calcul: list

    >>> from pyromaths.outils import Priorites3
    >>> c = ['-6', '**', '(-11)', '*', '(-5)']
    >>> Priorites3.recherche_puissance(c)
    (0, 3)
    >>> p = ['-', 'Polynome([[-4, 2]], "x")', '**', '2', '+', '3']
    >>> Priorites3.recherche_puissance(p)
    (1, 4)

    :rtype: tuple
    """
    if calcul.count("**"):
        i = calcul.index("**")
        # Si le calcul commence par '**', il n'y a rien à faire
        if i: return (i - 1, i + 2)
    return None

def recherche_operation(calcul, op, l_interdit, debut=0):
    """**recherche_operation**\ (*calcul*\ , *op*\ , *l_interdit*\ [, *debut*])

    Recherche la première occurrence de l'opération op dans la liste de
    calculs list. L'opérateur doit être entouré de deux nombres, le premier ne
    doit pas être précédé d'un opérateur inclus dans la liste l_interdit.
    L'opérateur à trouver doit être avant la position debut (si debut est non
    nul).

    :param calcul: le calcul à traiter
    :type calcul: list
    :param op: l'opérateur à chercher
    :type op: string
    :param l_interdit: liste des opérateurs à ignorer, pour respecter les priorités
    :type l_interdit: list
    :param debut: À partir d'où chercher l'opérateur dans l'expression
    :type debut: integer

    >>> from pyromaths.outils import Priorites3
    >>> c = ['-6', '**', '(-11)', '*', '(-5)']
    >>> Priorites3.recherche_operation(c, "*", ("**", "/"))
    None
    >>> c = ['-9', '-', '6', '*', '2', '-', '4']
    >>> Priorites3.recherche_operation(c, "*", ("**", "/"))
    3

    :rtype: integer
    """
    ind, fin = 0, len(calcul)
    for dummy in range(calcul.count(op)):
        ind = calcul.index(op, ind)
        if ind == 1 and ind < fin - 1 and EstNombre(calcul[ind - 1]) and\
            EstNombre(calcul[ind + 1]):
                if debut: debut = min(debut, ind)
                else: debut = ind
                break
        elif ind > 1 and ind < fin - 1 and EstNombre(calcul[ind - 1]) and\
            EstNombre(calcul[ind + 1]) and calcul[ind - 2] not in l_interdit:
                if debut: debut = min(debut, ind)
                else: debut = ind
                break
        else:
            ind += 1
    if debut: return debut
    else: return None

def recherche_fin_operation(calcul, op, l_interdit, debut):
    """**recherche_fin_operation**\ (*calcul*\ , *op*\ , *l_interdit*\ , *debut*)

    Recherche la fin de la suite d'opération @op situé à la position @debut
    de la liste de calculs @calcul, en vérifiant que cet enchaînement n'est pas
    suivi par un opérateur compris dans @l_interdit.

    :param calcul: le calcul à traiter
    :type calcul: string
    :param op: l'opérateur à chercher
    :type op: string
    :param l_interdit: liste des opérateurs à ignorer, pour respecter les priorités
    :type l_interdit: list
    :param debut: À partir d'où chercher l'opérateur dans l'expression
    :type debut: integer

    >>> from pyromaths.outils import Priorites3
    >>> c     = ['-9', '-', '6', '*', '2', '*', '5', '-', '4']
    >>> Priorites3.recherche_fin_operation(c, "*", ("**", "/"),0)
    2
    >>> Priorites3.recherche_fin_operation(c, "*", ("**", "/"),4)
    6

    :rtype: integer
    """
    index, fin = debut + 2, len(calcul)
    while index < fin - 1 and calcul[index] == op and EstNombre(calcul[index + 1]):
        index += 2
    if index < fin and calcul[index] in l_interdit:
        index -= 2
    if index == debut: return None
    return index

def recherche_produit(calcul):
    """**recherche_produit**\ (*calcul*)

    Recherche la première occurrence d'une multiplication ou divisions dans
    une liste de calculs. Elle doit être précédée d'un nombre qui ne doit pas
    être précédé d'une puissance ou d'une division, ni suivie d'un exposant.

    :param calcul: le calcul à traiter
    :type calcul: list

    >>> from pyromaths.outils import Priorites3
    >>> c = ['-9', '-', '6', '*', '2', '*', '5', '-', '4']
    >>> Priorites3.recherche_produit(c)
    (2, 7)

    :rtype: tuple
    """
    debut = recherche_operation(calcul, "*", ("**", "/"))
    debut = recherche_operation(calcul, "/", ("**", "/"), debut)
    if debut is not None:
        fin = recherche_fin_operation(calcul, calcul[debut], ("**"), debut)
        if fin is not None: return (debut - 1, fin)
    return None

def recherche_somme(calcul):
    """**recherche_somme**\ (*calcul*)

    Recherche la première occurrence d'une addition ou soustraction dans une
    liste de calculs. Elle doit être précédée d'un nombre qui ne doit pas être
    précédé d'une puissance, d'une multiplication, d'une division ou d'une
    soustraction, ni suivie d'un exposant, d'une multiplaication ou d'une
    division.

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.recherche_somme(['1', '-', '2', '-', '3'])
    (0, 5)
    >>> Priorites3.recherche_somme(['Polynome("x+1")', '-', 'Polynome("x+2")', '-', 'Polynome("x+3")'])
    (0, 5)

    :param calcul: le calcul à traiter
    :type calcul: list
    :rtype: tuple
    """
    debut = recherche_operation(calcul, "+", ("**", "*", "/", "-"))
    debut = recherche_operation(calcul, "-", ("**", "*", "/", "-"), debut)
    if debut is not None:
        fin = recherche_fin_operation(calcul, calcul[debut], ("**", "*", "/"),
                debut)
        if fin is not None: return (debut - 1, fin)
    return None

def recherche_neg(calcul):
    """**recherche_neg**\ (*calcul*)

    Recherche la première occurence d'un opposé `-(-5)` ou `-Polynome` ou
    d'une écriture du style `+(-5)` dans une liste de calculs qui ne doit pas être
    suivie d'un exposant.

    :param calcul: le calcul à traiter
    :type calcul: list
    :rtype: tuple
    """
    ind, debut, fin = 0, None, len(calcul)
    for dummy in range(calcul.count("-")):
        ind = calcul.index("-", ind)
        if ind < fin - 1 and EstNombre(calcul[ind + 1]):
            if calcul[ind + 1][0] == "(" or ind == 0:  # or isinstance(n, Polynome):
#                or (isinstance(n, Polynome) and (len(n)>1 or n.monomes[0][0]<0)):
                debut = ind
                break
        else:
            ind += 1
    for dummy in range(calcul[ind + 1:].count("+")):
        ind = calcul.index("+", ind)
        if ind < fin - 1 and EstNombre(calcul[ind + 1]) and calcul[ind + 1][0] == "(":
            if debut is not None: debut = min(debut, ind)
            else: debut = ind
            break
        else:
            ind += 1
    if debut is not None and (debut + 2 == fin or (debut + 2 < fin and calcul[debut + 2] != "**")):
        return (debut, debut + 2)
    else:
        return None

def effectue_calcul(calcul):
    """**effectue_calcul**\ (*calcul*)

    Effectue une étape du calcul en respectant les priorités

    :param calcul: le calcul à traiter
    :type calcul: list

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.effectue_calcul(['-5', '-', '(', '(-6)', '-', '1', '+', '(-3)', ')', '*', '(-1)'])
    ['-5', '-', '(', '-7', '-', '3', ')', '*', '(-1)']
    >>> Priorites3.effectue_calcul(['-5', '-', '(', '-7', '-', '3', ')', '*', '(-1)'])
    ['-5', '-', '-10', '*', '(-1)']
    >>> Priorites3.effectue_calcul(['-5', '-', '-10', '*', '(-1)'])
    ['-5', '-', '10']
    >>> Priorites3.effectue_calcul(['-5', '-', '10'])
    ['-15']
    >>> Priorites3.effectue_calcul(['-', 'Polynome("x-1")', '*', '2'])
    ['-', '(', 'Polynome([[2, 1]], "x", 0)', '+', 'Polynome([[-2, 0]], "x", 0)', ')']
    >>> Priorites3.effectue_calcul(['4', '*', 'Polynome("x+1")', '**', '2'])
    ['4', '*', '(', 'Polynome([[1, 1]], "x", 0)', '**', '2', '+', '2', '*', 'Polynome([[1, 1]], "x", 0)', '*', 'Polynome([[1, 0]], "x", 0)', '+', 'Polynome([[1, 0]], "x", 0)', '**', '2', ')']

    :rtype: list
    """
    from pyromaths.classes.PolynomesCollege import Polynome
    from pyromaths.classes.Fractions import Fraction
    from pyromaths.classes.SquareRoot import SquareRoot
    serie = (recherche_parentheses, recherche_puissance, recherche_produit,
            recherche_neg, recherche_somme)
    result, post, break_somme = [], "", False
    for recherche in serie:
        if break_somme and recherche == recherche_somme: break
        if calcul: test = recherche(calcul)
        else: test = None
        while test:
            pre = calcul[:test[0]]
            calc = calcul[test[0]:test[1]]
            post = calcul[test[1]:]
            # On essaie d'utiliser les priorités sur la première partie du
            # calcul pour raccourcir la résolution de l'expression
            if test != recherche_neg and pre:
                # Si on simplifie les écritures, on n'essaie pas les sommes
                tmp = effectue_calcul(pre)
                if tmp and result and result[-1] not in "+-**/(" and tmp[0][0]\
                    not in "+-*/)":
                    # un signe + si l'expression en a besoin
                    result.append("+")
                # On ajoute le résultat ou l'expression
                if tmp: result.extend(tmp)
                else: result.extend(pre)
            else:
                result.extend(pre)
            if recherche == recherche_parentheses:
                # On supprime les parenthèses autour de l'expression
                calc = calc[1:-1]
                # On réduit directement les calculs numériques dans une
                # expression littérale
                "Effectue les calculs entre parenthèses et remet les parenthèses si l'expression est de longueur supérieure à 1"
                sol = effectue_calcul(calc)
                if len(sol) > 1:
                    sol.insert(0, "(")
                    sol.append(")")
                elif sol and isinstance(eval(sol[0]), (int, float, Fraction)) and post and post[0] == "*" and \
                        _('Polynome(') in post[1] and len(eval(post[1])) == 1 and eval(post[1])[0][0] == 1:
                            # Sans doute une factorisation de sommes de polynômes
                            sol = [repr(eval(sol[0]) * eval(post[1]))]
                            post = post[2:]
                            # Suppression des parenthèses autour de ((9.0-80.0)*Polynome("x")) devenu (Polynome("-71.0x"))
                            if post and result and post[0] == ")" and result[-1] == "(" :
                                result, post = result[:-1], post[1:]

            else:
                if recherche == recherche_somme:
                    # Permet les cas 1 + Fraction(1, 2) + 1
                    # ou 3 + Polynome("5x") + 4
                    frac, poly, nombres = False, False, []
                    for i in range(0, len(calc), 2):
                        nombres.append(eval(calc[i]))
                        if isinstance(nombres[-1], Fraction): frac = True
                        elif isinstance(nombres[-1], Polynome): poly, var, details = True, nombres[-1].var, nombres[-1].details
                    if poly: nombres = [(Polynome([[i, 0]], var, details) , i)[isinstance(i, Polynome)] for i in nombres]
                    elif frac: nombres = [(Fraction(i, 1), i)[isinstance(i, Fraction)] for i in nombres]
                    if poly: classe = Polynome
                    elif frac: classe = Fraction
                    if poly or frac:
                        if calc[1] == '+': operation = classe.__add__
                        else: operation = classe.__sub__
                        if isinstance(nombres[0], (int, float)):
                            sol = operation(classe(nombres[0]), *nombres[1:])
                        else:
                            sol = operation(nombres[0], *nombres[1:])
                    else: sol = eval("".join(calc))
                elif recherche == recherche_produit and calc[1] == "*":
                    frac, poly, nombres = False, False, []
                    for i in range(0, len(calc), 2):
                        nombres.append(eval(calc[i]))
                        if isinstance(nombres[-1], Fraction): frac = True
                        elif isinstance(nombres[-1], Polynome): poly, var, details = True, nombres[-1].var, nombres[-1].details
                    if poly: nombres = [(Polynome([[i, 0]], var, details) , i)[isinstance(i, Polynome)] for i in nombres]
                    elif frac: nombres = [(Fraction(i, 1), i)[isinstance(i, Fraction)] for i in nombres]
                    if poly: classe = Polynome
                    elif frac: classe = Fraction
                    if poly or frac:
                        if isinstance(nombres[0], (int, float)):
                            sol = classe.__mul__(classe(nombres[0]), *nombres[1:])
                        else:
                            sol = classe.__mul__(nombres[0], *nombres[1:])
                    else: sol = eval("".join(calc))
                elif recherche == recherche_puissance:
                    sol = eval('(' + calc[0] + ')**' + calc[2])
                else:
                    sol = eval("".join(calc))
                if isinstance(sol, str): sol = splitting(sol)
                elif isinstance(sol, (int, float)): sol = [str(sol)]
                elif isinstance(sol, (Polynome, Fraction, SquareRoot)): sol = [repr(sol)]
                else :
                    raise ValueError(_(u"Le résultat %s a un format inattendu") % sol)
            if recherche == recherche_neg and (pre or result):
                # Ajoute le "+" ou sépare le "-":
                # "1-(-9)" => "1 + 9" et "1+(-9)" => "1 - 9"
                if sol[0][0] == "-": sol = ["-", sol[0][1:]]
                else: sol = ["+", sol[0]]
            #===================================================================
            # if recherche == recherche_produit and len(sol) > 1:
            #     # Ajoute les parenthèses si produit précédé d'un "-" ou "*"
            #     # ou suivi d'un "*"
            #===================================================================
            if len(sol) > 1 and sol[0] != "(" and sol[-1] != ")":
                # Ajoute les parenthèses si le résultat est précédé d'un "-" ou "*"
                # ou suivi d'un "*"
                if (result and result[-1] in "*-") or (pre and pre[-1] in "*-") or (post and post[0] == "*"):
                    sol.insert(0, "(")
                    sol.append(")")
            # Si @sol est négatif et @result se termine par un "+", on l'enlève
            if result and result[-1] == "+" and sol and sol[0][0] == "-":
                    result[-1] = "-"
                    sol[0] = sol[0].lstrip("-")
            result.extend(sol)
            calcul = post
            if calcul: test = recherche(calcul)
            else: test = None
            if post and recherche == recherche_neg: break_somme = True
    result.extend(post)
    if not result: result = calcul
    return result

def priorites(calcul):
    r"""**priorites**\ (*calcul*)

    Effectue un enchaînement d'opérations contenues dans calcul en
    respectant les priorités et en détaillant les étapes.

    :param calcul: le calcul à traiter
    :type calcul: string

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.priorites('-1+5-(-5)+(-6)*1')
    [['4', '+', '5', '-', '6'], ['9', '-', '6'], ['3']]
    >>> Priorites3.priorites('-5**2+6')
    [['-', '25', '+', '6'], ['-25', '+', '6'], ['-19']]
    >>> Priorites3.priorites('Polynome([[Fraction(6, 7), 0]], "x")*Polynome([[Fraction(1,3), 1], [1,0]], "x")')
    [['Polynome([[Fraction(6, 21, "s"), 1]], "x", 0)', '+', 'Polynome([[Fraction(6, 7), 0]], "x", 0)'], ['Polynome([[Fraction(2, 7), 1], [Fraction(6, 7), 0]], "x", 0)']]
    >>> Priorites3.priorites('-Fraction(-6,1)/Fraction(-4,1)')
    [['-', '(', 'Fraction(-6, 1)', '*', 'Fraction(1, -4)', ')'], ['-', 'Fraction("2*-3", "2*-2", "s")'], ['Fraction(-3, 2)']]

    :rtype: list
    """
    calcul = splitting(calcul)
    #calcul = split_calcul(calcul)
    solution = []
    while len(calcul) > 1:
        s = effectue_calcul(calcul)
        if s:
            if s == calcul: s, solution[0]
            solution.append(s)
        calcul = list(s)  # dissocie calcul et s
    if 'Polynome(' in calcul[0][:9]:
        from pyromaths.classes.PolynomesCollege import Polynome
        from pyromaths.classes.Fractions import Fraction
        p = eval(calcul[0])
        p = p.nreduction()
        while (solution and repr(p) != solution[-1][0]) or (not solution and repr(p) != calcul[0]):
            solution.append([repr(p)])
            p = p.nreduction()
    if 'Fraction(' in calcul[0][:9]:
        from pyromaths.classes.Fractions import Fraction
        from pyromaths.classes.SquareRoot import SquareRoot
        f = eval(calcul[0])
        f = f.traitement(True)
        while (solution and repr(f) != solution[-1][0]) or (not solution and repr(f) != calcul[0]):
            solution.append([repr(f)])
            # si la fraction est écrite sous la forme d'un entier, c'est fini
            if isinstance(f, str): f = eval(f)
            if isinstance(f, int): break
            f = f.traitement(True)
    return solution

def texifystring(calcul):
    r"""**texify**\ (*calcul*)

    convertit la chaîne de caractères en chaîne au format TeX.
    """
    return texify([splitting(calcul)])[0]

def texify(liste_calculs):
    r"""**texify**\ (*liste_calculs*)

    Convertit la liste de chaînes de caractères `liste_calculs` contenant
    des polynômes en liste de chaînes de caractères au format TeX

    **TODO :** intégrer cela dans :mod:`outils.Affichage` et gérer l'ensemble des
    classes de Pyromaths.

    :param calcul: le calcul à traiter
    :type calcul: string

    >>> from pyromaths.outils import Priorites3
    >>> from pyromaths.classes.PolynomesCollege import Polynome
    >>> l = [['4', '+', '5', '-', '6'], ['9', '-', '6'], ['3']]
    >>> Priorites3.texify(l)
    ['4+5-6', '9-6', '3']
    >>> Priorites3.texify(Priorites3.priorites('(-7)+8-Polynome([[-4, 1], [-9, 2], [-5, 0]], "x")'))
    ['1-\\left( -4\\,x-9\\,x^{2}-5\\right) ', '1+4\\,x+9\\,x^{2}+5', '9\\,x^{2}+4\\,x+6']
    >>> Priorites3.texify([['Fraction(5,6)', '**', '2']])
    ['\\left(  \\dfrac{5}{6} \\right) ^{2}']

    :rtype: list
    """
    from pyromaths.classes.PolynomesCollege import Polynome
    from pyromaths.classes.Fractions import Fraction
    from pyromaths.classes.SquareRoot import SquareRoot
    from .Affichage import decimaux
    ls = []
    enluminures = {'indice': r'_{', 'cancel':r'\cancel{'}
    isEnlumine = {'indice': False, 'cancel':False}
    for calcul in liste_calculs:
        if isinstance(calcul, str): calcul = splitting(calcul)
        s = ""
        puiss = 0
        for index in range(len(calcul)):
            el = calcul[index]
            for cle in list(isEnlumine.keys()):
                if isEnlumine[cle] and (not isinstance(el, list) or el[1] != cle):
                    s += '}'
                    isEnlumine[cle] = False
            if isinstance(el, list):
                if not isEnlumine[el[1]]:
                    s += enluminures[el[1]]
                    isEnlumine[el[1]] = True
                el = el[0]
            if el[:9] == "Polynome(":
                # Doit-on entourer ce polynôme de parenthèses ?
                p = eval(el)
                if index + 1 < len(calcul): q = calcul[index + 1]
                else: q = ""
                """ 3 cas :
                * {*,-}(2x+3) ou {*,-}(-2x)
                * (2x+3)*...
                * (2x+1)**2"""
                if type(p.monomes[0][0])==str:
                    a=eval(priorites(p.monomes[0][0])[0][-1])
                else:
                    a=p.monomes[0][0]
                if (s and s[-1] in "*-" and (len(p) > 1 or a < 0)) \
                    or (q and q == "*" and len(p) > 1) \
                    or ((len(p) > 1 or (a != 1 and p.monomes[0][1] > 0) or \
                         a < 0 or \
                         (a != 1 and isinstance(a, Fraction) and a.d != 1)) and q and q == "**"):
                    s += "(" + str(p) + ")"
                elif s and s[-1] == "+" and type(a) == int and a < 0:
                    s = s[:-1]
                    s += str(p)
                else:
                    s += str(p)
            elif el[:9] == "Fraction(":
                # Doit-on entourer cette fraction de parenthèses ?
                p = splitting(el[9:-1])
                # Gère le cas ou la fraction comprend des objets SquareRoot
                t = [""]
                for i in range(len(p)):
                    if p[i] == ',': t.append("")
                    else: t[-1] += p[i]
                p = t
                if len(p) == 2:
                    # texfrac = str(Fraction(eval(p[0]), eval(p[1])))
                    texfrac = str(Fraction(p[0], p[1]))
                else:
                    texfrac = str(Fraction(p[0], p[1], p[2]))
                if index + 1 < len(calcul): q = calcul[index + 1]
                else: q = ""
                if (eval(p[0]) < 0 or p[1] != "1") and q == "**":
                    s += "( " + texfrac + " )"
                else:
                    s += texfrac
            elif el[:11] == "SquareRoot(":
                p = eval(el)
                if index + 1 < len(calcul): q = calcul[index + 1]
                else: q = ""
                """ 3 cas :
                * {*,-}(2x+3) ou {*,-}(-2x)
                * (2x+3)*...
                * (2x+1)**2"""
                if (s and s[-1] in "*-" and (len(p) > 1 or p[0][0] < 0)) \
                    or (q and q == "*" and len(p) > 1) \
                    or ((len(p) > 1 or (p[0][0] != 1 and p[0][1] > 0) or \
                         p[0][0] < 0 or \
                         (p[0][0] != 1 and isinstance(p[0][0], Fraction) and p[0][0].d != 1)) and q and q == "**"):
                    s += "(" + str(p) + ")"
                elif s and s[-1] == "+" and p[0][0] < 0:
                    s = s[:-1]
                    s += str(p)
                else:
                    s += str(p)
            elif EstNombre(el):
                if index + 1 < len(calcul): q = calcul[index + 1]
                else: q = ""
                if el[0] == "(": s += "(" + decimaux(el[1:-1]) + ")"

                elif el[0] == '-' and ((s and s[-1] in "+/*-") \
                    or (q and q == "**")):
                    s += "(" + decimaux(el) + ")"

                else: s += decimaux(el)
            elif el == "**":
                s += "**{"
                puiss = 1
            elif el == "(":
                if puiss: puiss += 1
                s += "("
            elif el == ")":
                if puiss: puiss -= 1
                s += ")"
            else :
                # "+", "-", "*", "/"
                s += el
            if puiss == 1 and s[-1] != r"{":
                puiss = 0
                s += "}"
        for cle in list(isEnlumine.keys()):
            if isEnlumine[cle] and (not isinstance(el, list) or el[1] != cle):
                s += '}'
        s = s.replace("**{", "^{")
        s = s.replace("(", "\\left( ")
        s = s.replace(")", "\\right) ")
        s = s.replace("\\left\\left", "\\left ")
        s = s.replace("\\right\\right", "\\right ")
        s = s.replace("*", "\\times ")
        s = s.replace("/", "\\div ")
        if not ls or s != ls[-1]:
            ls.append(s)
    return ls

def plotify(calcul):
    r"""**plotify**\ (*calcul*)

    Convertit la chaîne de caractères `calcul` contenant
    des polynômes une chaîne de caractères au format psplot

    **TODO :** intégrer cela dans :mod:`outils.Affichage` et gérer l'ensemble des
    classes de Pyromaths.

    :param calcul: le calcul à traiter
    :type calcul: string

    >>> from pyromaths.outils import Priorites3
    >>> Priorites3.plotify('Polynome([[Fraction(-5, 192), 4], [Fraction(2, 96), 3], [Fraction(41, 48), 2], [Fraction(-7, 12), 1], [-4, 0]], "x", False)')
    -5/192*x^4+2/96*x^3+41/48*x^2-7/12*x^1-4

    :rtype: str
    """
    from pyromaths.classes.PolynomesCollege import Polynome
    from pyromaths.classes.Fractions import Fraction
    from .Affichage import decimaux

    if isinstance(calcul, str): calcul = splitting(calcul)
    s = ""
    puiss = 0
    for i in range(len(calcul)):
        el = calcul[i]
        if el[:9] == "Polynome(":
            # Doit-on entourer ce polynôme de parenthèses ?
            p = eval(el)
            if i + 1 < len(calcul): q = calcul[i + 1]
            else: q = ""
            """ 3 cas :
            * {*,-}(2x+3) ou {*,-}(-2x)
            * (2x+3)*...
            * (2x+1)**2"""
            res = []
            for m in p.monomes:
                texte = ''
                if isinstance(m[0], Fraction): texte = str(m[0].n) + "/" + str(m[0].d)
                elif isinstance(m[0], (float, int)): texte = str(m[0])
                if m[1]: texte += "*x^%s" % m[1]
                res.append(texte)
            t = "+".join(res)
            t = t.replace("+-", "-")
            if (s and s[-1] in "*-" and (len(p) > 1 or p.monomes[0][0] < 0)) \
                or (q and q == "*" and len(p) > 1) \
                or ((len(p) > 1 or p.monomes[0][0] != 1) and q and q == "**"):
                s += "(" + t + ")"
            elif s and s[-1] == "+" and p.monomes[0][0] < 0:
                s = s[:-1]
                s += t
            else:
                s += t
        elif el[:9] == "Fraction(":
            # Doit-on entourer cette fraction de parenthèses ?
            p = el[9:-1].split(",")
            if len(p) == 2:
                texfrac = p[0] + "/" + p[1]
            else:
                raise ValueError(_(u'On ne devrait pas rencontrer de fraction non simplifiée ici'))
            if i + 1 < len(calcul): q = calcul[i + 1]
            else: q = ""
            if (eval(p[0]) < 0 or p[1] != "1") and q and q == "**":
                s += "( " + texfrac + " )"
            else:
                s += texfrac
        elif EstNombre(el):
            if el[0] == "(": s += "(" + decimaux(el[1:-1]) + ")"
            else: s += decimaux(el)
        elif el == "**":
            s += "^("
            puiss += 1
        elif el == "(":
            if puiss: puiss += 1
            s += "("
        elif el == ")":
            if puiss: puiss -= 1
            s += ")"
        else :
            # "+", "-", "*", "/"
            s += el
        if puiss == 1 and s[-1] != "":
            puiss = 0
            s += ")"
    s = s.replace("**{", "^(")
    s = s.replace(r"\,", "*")
    return s
